Package: rapidsplithalf
Type: Package
Title: A Fast Permutation-Based Split-Half Reliability Algorithm
Version: 0.8
Authors@R: person("Sercan", "Kahveci", role = c("aut","cre"),email = "sercan.kahveci@plus.ac.at",comment=c(ORCID = "0000-0002-4139-5710"))
Description: Accurately estimates the reliability of cognitive tasks using a fast and flexible permutation-based split-half reliability algorithm that supports stratified splitting while maintaining equal split sizes. See Kahveci, Bathke, and Blechert (2025) <doi:10.3758/s13423-024-02597-y> for details.
License: GPL (>= 2)
BugReports: https://github.com/Spiritspeak/rapidsplit/issues/
Depends: R(>= 4.1.0)
Imports: Rcpp (>= 1.0.5), fastmatch, kit
Suggests: knitr, rmarkdown
LinkingTo: Rcpp
RoxygenNote: 7.3.3
Encoding: UTF-8
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-07-15 14:43:27 UTC; b1066151
Author: Sercan Kahveci [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4139-5710>)
Maintainer: Sercan Kahveci <sercan.kahveci@plus.ac.at>
Repository: CRAN
Date/Publication: 2026-07-15 15:10:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-07-15 15:23:04 UTC; unix
Archs: rapidsplithalf.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,rapidsplit)
S3method(print,compcorr)
S3method(print,rapidsplit)
export(bootstrapWeights)
export(colMeansMasked)
export(colMedians)
export(colMediansMasked)
export(colProds)
export(colSds)
export(colSdsMasked)
export(compcorr)
export(comprel)
export(corByColumns)
export(corByColumns_mask)
export(corStatsByColumns)
export(cormean)
export(excludeOutliersByMask)
export(generateSplits)
export(loo.rapidsplit)
export(maskOutliers)
export(maskOutliersMasked)
export(meansByMask)
export(meansByWeight)
export(mediansByMask)
export(mediansByWeight)
export(r2p)
export(r2t)
export(r2z)
export(rapidsplit)
export(rapidsplit.chunks)
export(rconfint)
export(requiredTestSize)
export(sdsByMask)
export(sdsByWeight)
export(spearmanBrown)
export(stratifiedItersplits)
export(t2r)
export(z2r)
import(Rcpp)
importFrom(fastmatch,"%fin%")
importFrom(fastmatch,fmatch)
importFrom(grDevices,rgb)
importFrom(graphics,text)
importFrom(kit,countOccur)
importFrom(kit,funique)
importFrom(kit,uniqLen)
importFrom(stats,cor)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(rapidsplithalf)

Readme for {rapidsplithalf}
================
Sercan Kahveci
2024-06-25

[![CRAN
Version](http://www.r-pkg.org/badges/version/rapidsplithalf)](https://cran.r-project.org/package=rapidsplithalf)
[![Downloads](https://cranlogs.r-pkg.org/badges/rapidsplithalf)](https://cran.r-project.org/package=rapidsplithalf)

## Installation

    if(!require(remotes)){ install.packages("remotes") }
    remotes::install_github("Spiritspeak/rapidsplit")

## Introduction

The R package *rapidsplithalf* computes the permutated split-half
reliability of reaction time tasks quickly and accurately, using a
balanced splitting algorithm that ensures both halves are of maximally
equal size regardless of how many subdivisions exist, for example due to
conditions or stratification. As far as we know, this is the only R
package for split-half reliability that does so. *rapidsplit* further
allows for stratification of the splits, for example by stimulus, to
ensure that the two halves are as similar to each other as possible.
[Kahveci et al. (2022)](https://doi.org/10.31234/osf.io/ta59r) found
that this stratification enhances the accuracy of the reliability
estimate. Lastly, there is support for performing certain pre-processing
operations *after* splitting the data rather than before.

## Getting started

``` r
library(rapidsplithalf)
data(foodAAT)
data(raceIAT)
```

Let’s say we have a reaction time task with 2 conditions whose RTs we
average and then subtract from each other within each participant. We
want to know the reliability of this difference score. What do we do?

[Kahveci et al. (2022)](https://doi.org/10.31234/osf.io/ta59r) give a
number of recommendations based on simulation analyses.

- Split-half reliability far exceeds in precision compared to Cronbach’s
  alpha. They also find much better accuracy in performing many repeated
  random splits and averaging the result, rather than performing a
  single even-odd split.

- Most common studies need up to 5500 splits to accurately detect the
  split-half reliability.

- The accuracy of the reliability estimate is further improved by
  stratifying the random splits by stimulus such that both halves
  feature a near-equal number of trials for each stimulus, thereby
  ensuring that the two halves resemble replications of each other.

- They recommend computing the reliability of the score that is actually
  used in analyses of the manuscript, i.e. if a difference score is used
  in correlational analyses, then the reliability of that difference
  score should be computed rather than of the means that were subtracted
  from each other. Likewise, if the data was aggregated using medians,
  then the reliability of a median-based score should be computed.

### Mean difference score

``` r
rel<-
rapidsplit(data=foodAAT,
           subjvar="subjectid", # Subject identifier
           diffvars="is_pull", # The variable indicating the two conditions that need to be subtracted from each other
           stratvars="stimid", # splits are stratified by stimulus ID such that each half features a near-equal number of trials for each stimulus
           aggvar="RT", # Defining the variable to be aggregated
           aggfunc="means", # Method to aggregate each condition: simple means
           splits=5500)
```

    ## Generating splits...
    ## ================================================================================
    ## Aggregating conditions...
    ## ================================================================================

``` r
print(rel)
```

    ## Full-length reliability (Spearman-Brown coefficient):
    ## rSB (34) = .44, 95%CI [.04, .7], p = .007

We can plot one of the splits from the reliability object to check for
outliers.

``` r
plot(rel)
```

<img src="man/figures/README-plotter-1.png" width="50%" />

Or we can plot all of the splits simultaneously in a single figure.

``` r
plot(rel,type="all")
```

<img src="man/figures/README-plotter2-1.png" width="50%" />

As you can see, the points fall into a diagonal line, with each line
representing a single participant. The participants with a longer line
have a more uncertain score.

### Double median difference score

As discussed, if you’re using medians to aggregate your conditions, then
the reliability of a medin-based score should be computed. We do so here
within the context of a double-difference score, common to the
approach-avoidance task. This has the following format: (push target -
pull target) - (push control - pull control)

``` r
rel2<-
rapidsplit(data=foodAAT,
           subjvar="subjectid",
           # We specify 2 diffvars for a double-difference score
           diffvars=c("is_pull","is_target"),
           stratvars="stimid",
           aggvar="RT",
           # We specify the median here
           aggfunc="medians",
           splits=5500)
```

    ## Generating splits...
    ## ================================================================================
    ## Aggregating conditions...
    ## ================================================================================

``` r
print(rel2)
```

    ## Full-length reliability (Spearman-Brown coefficient):
    ## rSB (34) = .87, 95%CI [.79, .93], p = 0

### IAT D-score

We can also compute the reliability of a D-score from an implicit
association task.

``` r
iatrel<-
  rapidsplit(data=raceIAT,
             subjvar="session_id",
             diffvars="congruent",
             
             # The subscorevar argument specifies that we want to compute
             # multiple scores for each participant, based on subsets of
             # their data, and then average the scores together 
             # as is done in the IAT D-score.
             subscorevar="blocktype",
             aggvar="latency",
             splits=1000,
             
             # the errorhandling argument controls how error trials
             # are replaced with the block mean plus a penalty,
             # as is done in the IAT D-score.
             errorhandling=list(type="fixedpenalty",
                                errorvar="error",
                                fixedpenalty=600,
                                blockvar="block_number"),
             
             # The standardize argument specifies that we want to
             # divide the person's score by the standard deviation of their
             # RTs, as in the IAT D-score.
             standardize=TRUE)
```

    ## Generating splits...
    ## ================================================================================
    ## Replacing error trials
    ## ================================================================================
    ## Aggregating conditions...
    ## ================================================================================
    ## Computing standard deviations to standardize scores by...
    ## ================================================================================

``` r
print(iatrel)
```

    ## Full-length reliability (Spearman-Brown coefficient):
    ## rSB (198) = .84, 95%CI [.76, .89], p = 0

``` r
plot(iatrel,show.labels=FALSE)
```

<img src="man/figures/README-iatrel-1.png" width="50%" />

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, 
                      fig.path = "./../man/figures/README-")

## ----intro--------------------------------------------------------------------
library(rapidsplithalf)
data(foodAAT)
data(raceIAT)

## ----singlediff---------------------------------------------------------------
rel<-
rapidsplit(data=foodAAT,
           subjvar="subjectid", # Subject identifier
           diffvars="is_pull", # The variable indicating the two conditions that need to be subtracted from each other
           stratvars="stimid", # splits are stratified by stimulus ID such that each half features a near-equal number of trials for each stimulus
           aggvar="RT", # Defining the variable to be aggregated
           aggfunc="means", # Method to aggregate each condition: simple means
           splits=5500)

print(rel)

## ----plotter,fig.asp=1--------------------------------------------------------
plot(rel)

## ----plotter2,fig.asp=1-------------------------------------------------------
plot(rel,type="all")

## ----doublediff---------------------------------------------------------------
rel2<-
rapidsplit(data=foodAAT,
           subjvar="subjectid",
           # We specify 2 diffvars for a double-difference score
           diffvars=c("is_pull","is_target"),
           stratvars="stimid",
           aggvar="RT",
           # We specify the median here
           aggfunc="medians",
           splits=5500)

print(rel2)

## ----iatrel,fig.asp=1---------------------------------------------------------

iatrel<-
  rapidsplit(data=raceIAT,
             subjvar="session_id",
             diffvars="congruent",
             
             # The subscorevar argument specifies that we want to compute
             # multiple scores for each participant, based on subsets of
             # their data, and then average the scores together 
             # as is done in the IAT D-score.
             subscorevar="blocktype",
             aggvar="latency",
             splits=1000,
             
             # the errorhandling argument controls how error trials
             # are replaced with the block mean plus a penalty,
             # as is done in the IAT D-score.
             errorhandling=list(type="fixedpenalty",
                                errorvar="error",
                                fixedpenalty=600,
                                blockvar="block_number"),
             
             # The standardize argument specifies that we want to
             # divide the person's score by the standard deviation of their
             # RTs, as in the IAT D-score.
             standardize=TRUE)

print(iatrel)

plot(iatrel,show.labels=FALSE)

#' @title rapidsplithalf package
#' @name rapidsplithalf
#' @aliases rapidsplithalf-package
#' @import Rcpp
#' @useDynLib rapidsplithalf
#' @importFrom stats cor setNames qnorm pt pnorm quantile weighted.mean
#' @importFrom grDevices rgb
#' @importFrom graphics text
#' @importFrom utils txtProgressBar setTxtProgressBar
#' @importFrom fastmatch fmatch %fin%
#' @importFrom kit funique countOccur uniqLen
#' @author Sercan Kahveci
#' @description
#' Accurately estimates the reliability of cognitive tasks using a fast and flexible 
#' permutation-based split-half reliability algorithm 
#' that supports stratified splitting while maintaining equal split sizes. 
#' See Kahveci, Bathke, and Blechert (2025) \doi{10.3758/s13423-024-02597-y} for details.
#' To learn more about rapidsplithalf, view the introductory vignette:
#' \code{vignette("rapidsplithalf",package="rapidsplithalf")}
#' 
NULL

#' Spearman-Brown correction
#' Perform a Spearman-Brown correction on the provided correlation score.
#' @param r To-be-corrected correlation coefficient.
#' @param ntests An integer indicating how many times larger the full test is, 
#' for which the corrected correlation coefficient is being computed.
#' @param fix.negative How will negative input values be dealt with? 
#' * \code{"mirror"} submits the absolute correlations to the formula 
#' and restores the original sign afterwards.
#' * \code{"nullify"} sets negative correlations to zero.
#' * \code{"none"} leaves them as-is (not recommended).
#' 
#' @details
#' When \code{ntests=2}, the formula will compute what the correlation coefficient would be 
#' if the test were twice as long.
#' 
#' @return Spearman-Brown corrected correlation coefficients.
#' @export
#' @author Sercan Kahveci
#' @md
#' @seealso [requiredTestSize()] to compute how many items a test with a known reliability needs, to achieve a goal reliability.
#' @examples
#' spearmanBrown(.5)
#' 
spearmanBrown<-function(r,ntests=2,fix.negative=c("mirror","nullify","none")){
  fix.negative<-match.arg(fix.negative)
  
  if(fix.negative=="mirror"){
    sb<-ntests*r / (1+(ntests-1)*abs(r))
  }else{
    # includes "none"
    sb<-ntests*r / (1+(ntests-1)*r)
    
    if(fix.negative=="nullify"){
      sb[sb<0]<-0
    }
  }
  return(sb)
}


#' Required test size for target reliability
#' 
#' Compute how many items a test needs, for its reliability to reach a goal value.
#'
#' @param obs_r The observed reliability
#' @param goal_r The goal reliability
#' @param n Current test size, i.e., the number of items or trials.
#' @param fix.negative How should negative observed reliabilities be handled? See [spearmanBrown()].
#'
#' @returns The required test size.
#' @details This function is obtained by solving the [spearmanBrown()] formula for the \code{ntests} argument.
#' @md
#' @author Sercan Kahveci
#' @export
#'
#' @examples
#' requiredTestSize(obs_r=0.5, goal_r=0.8,n=256)
#' 
#' requiredTestSize(obs_r=0, goal_r=0.8,n=256)
#' 
requiredTestSize<-function(obs_r,goal_r=.8,n=1,fix.negative=c("mirror","nullify","none")){
  fix.negative<-match.arg(fix.negative)
  if(fix.negative=="mirror"){
    k<-(1/obs_r-sign(obs_r))/(1/goal_r-sign(obs_r))*n
  }else{
    if(fix.negative=="nullify"){
      obs_r[obs_r<0]<-0
    }
    k<-(1/obs_r-1)/(1/goal_r-1)*n
  }
  k[obs_r<=0 & goal_r>0]<-Inf
  k[obs_r>=0 & goal_r<0]<-NA
  return(k)
}

# Helper functions used by rapidsplit() and associated methods.

mf<-function(x,digits=2){
  s<-format(round(x,digits=digits),
            digits=ifelse(digits!=0,digits,getOption("digits")),
            drop0trailing=T,scientific=F,nsmall=digits)
  s<-sub("(?<![0-9])0+(?=\\.)", "",s, perl = TRUE)
  return(s)
}

runlengths <- function (x) {
  n <- length(x)
  if (n == 0L) return(integer())
  y <- x[-1L] != x[-n]
  i <- c(which(y), n)
  return(diff(c(0L, i)))
}

cols2ids<-function(df){
  for(col in seq_along(df)){
    df[,col]<-as.numeric(as.factor(df[[col]]))
  }
  df
}

clamp.range<-function(x){
  x[x==min(x)]<- -1
  x[x==max(x)]<- 1
  x
}

# takes vector input
makeCustomAggregator<-function(fun){
  function(x,mask){
    out<-numeric(ncol(mask))
    for(i in seq_len(ncol(mask))){
      out[i]<-fun(x[mask[,i,drop=T]])
    }
    return(out)
  }
}

# takes matrix input
makeCustomAggregator2<-function(fun){
  function(x,mask){
    out<-numeric(ncol(mask))
    for(i in seq_len(ncol(mask))){
      out[i]<-fun(x[mask[,i,drop=T],i])
    }
    return(out)
  }
}

datachecker<-function(data,subjvar,diffvars,stratvars,subscorevar,aggvar,
                      errorhandling,standardize){
  if(!is.data.frame(data)){
    stop("Input dataset not a data.frame.")
  }
  if(length(subjvar)!=1L || length(aggvar)!=1L || !is.character(subjvar) || !is.character(aggvar)){
    stop("Arguments subjvar and aggvar must be character vectors of length 1.")
  }
  if(length(subscorevar)>1L){
    stop("Can only specify 1 subscorevar.")
  }else if(length(subscorevar)==1L){
    if(!is.character(subscorevar)){
      stop("Argument subscorevar must be a character vector.")
    }
  }
  if(length(diffvars)>0L){
    if(!is.character(diffvars)){
      stop("Argument diffvars must be a character vector.")
    }
  }
  if(length(stratvars)>0L){
    if(!is.character(stratvars)){
      stop("Argument stratvars must be a character vector.")
    }
  }
  if(length(aggvar)>1L){
    stop("Can only have one aggvar.")
  }else if(!is.numeric(data[[aggvar]])){
    stop("The variable specified by aggvar must be numeric.")
  }
  
  if(!all(c(subjvar,diffvars,stratvars,subscorevar,aggvar,
            errorhandling$blockvar,errorhandling$errorvar) %fin% names(data))){
    stop("Not all specified columns present in data.")
  }
  if(length(intersect(diffvars,stratvars))>0L){
    stop("Cannot have a variable specified as both diffvars and stratvars: ",
         intersect(diffvars,stratvars))
  }
  if(length(intersect(subscorevar,stratvars))>0L){
    stop("Cannot have a variable specified as both subscorevar and stratvars: ",
         intersect(subscorevar,stratvars))
  }
  if(length(intersect(diffvars,subscorevar))>0L){
    stop("Cannot have a variable specified as both diffvars and subscorevar: ",
         intersect(diffvars,subscorevar))
  }
  if(anyDuplicated(c(subjvar,diffvars,stratvars,aggvar,subscorevar,errorhandling$errorvar))){
    stop("Cannot have the same variable specified more than once among ",
         "subjvar, diffvars, stratvars, aggvar, subscorevar, or errorhandling$errorvar.")
  }
  
  if(anyNA(data[[aggvar]])){
    stop(sum(is.na(data[[aggvar]]))," missing values present in variable to be aggregated.",
         " Please remove them first.")
  }
  if(!is.numeric(data[[aggvar]])){
    stop("Aggregation variable is not numeric.")
  }
  nvalues<-sapply(data[,diffvars,drop=FALSE],uniqLen)
  if(any(nvalues!=2L)){
    stop("Less or more than 2 unique values present in diffvars ",diffvars[nvalues!=2L],
         ". The variables specified by this argument should feature two unique values, ",
         "indicating which condition (the lower value) ",
         "will be subtracted from the other (the higher value).")
  }
  
  if(length(subscorevar)>0L){
    condpairs<-funique(data[,c(subjvar,subscorevar,diffvars),drop=FALSE])
    subscoresubvalues<-countOccur(condpairs[,c(subjvar,subscorevar),drop=FALSE])
    smallsubscores<-subscoresubvalues$Count!=2^length(diffvars)
    if(any(smallsubscores)){
      stop("Some participants do not have data for all specified conditions within subscores: ",
           paste0("participant ",
                  subscoresubvalues[[subjvar]][smallsubscores],
                  " within subscore ",
                  subscoresubvalues[[subscorevar]][smallsubscores],
                  collapse=", "))
    }
    
    condcounts<-countOccur(data[,c(subjvar,subscorevar,diffvars),drop=FALSE])
    smallconds<-condcounts$Count<2L
    if(any(smallconds)){
      stop("Insufficient data (<2 obs) in 1 or more conditions belonging to these participants: ",
           paste0(funique(paste0(condcounts[[subjvar]][smallconds],
                                 " within subscore ",
                                 condcounts[[subscorevar]][smallconds])),
                  collapse=", "))
    }
    
    if(standardize){
      datapersubject<-countOccur(data[,c(subjvar,subscorevar),drop=FALSE])
      if(!all(datapersubject$Count>=4L)){
        smallsubscores <- datapersubject$Count<4L
        stop("Cannot compute some participants' standard deviation due to insufficient data: ",
             paste(datapersubject[smallsubscores,subjvar],
                   "within subscore",
                   datapersubject[smallsubscores,subscorevar],collapse=", "))
      }
      
      indices<-do.call(paste,data[,c(subjvar,subscorevar),drop=FALSE])
      nuniques<-tapply(X=data[[aggvar]],INDEX=indices,FUN=uniqLen)
      toofew<-names(nuniques[nuniques==1L])
      if(any(nuniques==1L)){
        stop("Cannot compute the SD of every participant/subscore, ",
             "due to less than 2 unique values in these participants/subscores: ",
             paste0(funique(paste0("participant ",data[[subjvar]][indices %fin% toofew],
                                   " within subscore ",data[[subscorevar]][indices %fin% toofew])),
                    collapse=", "))
      }
    }
  }else{
    condpairs<-funique(data[,c(subjvar,diffvars),drop=FALSE])
    subscoresubvalues<-countOccur(condpairs[[subjvar]])
    smallsubscores<-subscoresubvalues$Count!=2L^length(diffvars)
    if(any(smallsubscores)){
      stop("Some participants do not have data for all specified conditions: ",
           paste0("participant ",
                  subscoresubvalues[[subjvar]][smallsubscores],
                  " within subscore ",
                  subscoresubvalues[[subscorevar]][smallsubscores],
                  collapse=", "))
    }
    
    condcounts<-countOccur(data[,c(subjvar,diffvars),drop=FALSE])
    smallconds<-condcounts$Count<2L
    if(any(smallconds)){
      stop("Insufficient data (<2 obs) in 1 or more conditions belonging to these participants: ",
           paste(funique(condcounts[[subjvar]][smallconds]),collapse=", "))
    }
    
    datapersubject<-countOccur(data[[subjvar]])
    if(standardize && !all(datapersubject$Count>=4L)){
      stop("Cannot compute some participants' standard deviation due to insufficient data: ",
           paste(datapersubject$Variable[datapersubject$Count<4L],collapse=", "))
    }
    if(standardize){
      nuniques<-tapply(X=data[[aggvar]],INDEX=data[[subjvar]],FUN=uniqLen)
      toofew<-names(nuniques[nuniques==1L])
      if(any(nuniques==1L)){
        stop("Cannot compute the SD of every participant/subscore, ",
             "due to less than 2 unique values in these participants: ",
             paste0(toofew,collapse=", "))
      }
    }
  }
}

checkerrorhandling<-function(x){
  if(length(x$type)>1L){
    x$type<-x$type[1L]
  }
  if(length(x$type)==0L){
    x$type<-"none"
  }
  missings<-setdiff(c("errorvar","blockvar"),names(x))
  missings<-vector(mode="list",length=length(missings)) |> setNames(missings)
  x<-c(x,missings)
  if(length(x$fixedpenalty)==0L){
    x$fixedpenalty<-600
  }
  return(x)
}

#' rapidsplit
#' @description A very fast algorithm for computing stratified permutation-based split-half reliability.
#'
#' @param data Dataset, a \code{data.frame}.
#' @param subjvar Subject ID variable name, a \code{character}.
#' @param aggvar Name of variable whose values to aggregate, a \code{character}.
#' Examples include reaction times and error rates.
#' @param diffvars Names of variables that determine which conditions 
#' need to be subtracted from each other, \code{character}.
#' @param stratvars Additional variables that the splits should 
#' be stratified by; a \code{character}.
#' @param subscorevar A \code{character} variable identifying subgroups within 
#' a participant's data from which separate scores should be computed. 
#' To compute a participant's final score, these subscores will be averaged together. 
#' A typical use case is the D-score of the implicit association task.
#' @param splits Number of split-halves to average, an \code{integer}. 
#' It is recommended to use around 5000.
#' @param aggfunc The function by which to aggregate the variable 
#' defined in \code{aggvar}; can be \code{"means"}, \code{"medians"}, 
#' or a custom function (not a function name). 
#' This custom function must take a numeric vector and output a single value.
#' @param errorhandling A list with 4 named items, to be used to replace error trials
#'  with the block mean of correct responses plus a fixed penalty, as in the IAT D-score.
#'  The 4 items are \code{type} which can be set to \code{"none"} for no error replacement, 
#'  or \code{"fixedpenalty"} to replace error trials as described;
#'  \code{errorvar} requires name of the \code{logical} variable indicating an incorrect response
#'  (as \code{TRUE}); \code{fixedpenalty} indicates how much of a penalty should be added 
#'  to said block mean; and \code{blockvar} indicates the name of the block variable.
#' @param standardize Whether to divide by scores by the subject's SD; a \code{logical}. 
#' Regardless of whether error penalization is utilized, this standardization 
#' will be based on the unpenalized SD of correct and incorrect trials, as in the IAT D-score.
#' @param include.scores Include all individual split-half scores?
#' @param verbose Display progress bars? Defaults to \code{TRUE}.
#' @param check Check input for possible problems?
#'
#' @return A \code{list} containing 
#' 
#' * \code{r}: the averaged reliability.
#' 
#' * \code{ci}: the 95% confidence intervals.
#' 
#' * \code{allcors}: a vector with the reliability of each iteration.
#' 
#' * \code{nobs}: a vector with (1) the number of participants and (2) the average number of values per participant.
#' 
#' * \code{rcomponents}: a list containing the mean variance of the scores of both halves, 
#' as well as their mean covariance.
#' 
#' * \code{scores}: the individual participants scores in each split-half, 
#' contained in a list with two matrices (Only included if requested with \code{include.scores}).
#' 
#' @md
#' 
#' @note
#' * [rapidsplit()] function can use a lot of memory in one go. 
#' If you are computing the reliability of a large dataset or you have little RAM, 
#' it may pay off to use [rapidsplit.chunks()] instead.
#' 
#' * It is currently unclear it is better to pre-process your data before or after splitting it. 
#' If you are computing the IAT D-score, 
#' you can therefore use \code{errorhandling} and \code{standardize} to perform these two actions
#' after splitting, or you can process your data before splitting and forgo these two options.
#' 
#' @details
#' The order of operations (with optional steps between brackets) is: 
#' * Splitting
#' * (Replacing error trials within block within split)
#' * Computing aggregates per condition (per subscore) per person
#' * Subtracting conditions from each other
#' * (Dividing the resulting (sub)score by the SD of the data used to compute that (sub)score)
#' * (Averaging subscores together into a single score per person)
#' * Computing the covariances of scores from one half with scores from the other half 
#' for every split
#' * Computing the variances of scores within each half for every split
#' * Computing the average split-half correlation with the average variances and covariance 
#' across all splits, using [corStatsByColumns()]
#' * Applying the Spearman-Brown formula to the absolute correlation 
#' using [spearmanBrown()], and restoring the original sign after
#' 
#' [cormean()] was used to aggregate correlations in previous versions 
#' of this package & in the associated manuscript, but the method based on 
#' (co)variance averaging was found to be more accurate. This was suggested by 
#' prof. John Christie of Dalhousie University.
#' 
#' @references Kahveci, S., Bathke, A.C. & Blechert, J. (2024) 
#' Reaction-time task reliability is more accurately computed with 
#' permutation-based split-half correlations than with Cronbach’s alpha. 
#' Psychonomic Bulletin and Review. \doi{10.3758/s13423-024-02597-y}
#' 
#' @export
#' @author Sercan Kahveci
#' @examples 
#' 
#' data(foodAAT)
#' # Reliability of the double difference score:
#' # {RT(push food)-RT(pull food)} - {RT(push object)-RT(pull object)}
#' 
#' frel<-rapidsplit(data=foodAAT,
#'                  subjvar="subjectid",
#'                  diffvars=c("is_pull","is_target"),
#'                  stratvars="stimid",
#'                  aggvar="RT",
#'                  splits=100)
#'                  
#' print(frel)
#' 
#' plot(frel,type="average")
#' 
#'            
#' # Compute a single random split-half reliability of the error rate
#' rapidsplit(data=foodAAT,
#'            subjvar="subjectid",
#'            aggvar="error",
#'            splits=1,
#'            aggfunc="means")
#' 
#' # Compute the reliability of an IAT D-score
#' data(raceIAT)
#' rapidsplit(data=raceIAT,
#'            subjvar="session_id",
#'            diffvars="congruent",
#'            subscorevar="blocktype",
#'            aggvar="latency",
#'            errorhandling=list(type="fixedpenalty",errorvar="error",
#'                               fixedpenalty=600,blockvar="block_number"),
#'            splits=10,
#'            standardize=TRUE)
#' 
rapidsplit<-function(data,subjvar,aggvar,diffvars=NULL,stratvars=NULL,subscorevar=NULL,
                      splits=6000L,
                      aggfunc=c("means","medians"),
                      errorhandling=list(type=c("none","fixedpenalty"),
                                         errorvar=NULL,fixedpenalty=600,blockvar=NULL),
                      standardize=FALSE,include.scores=TRUE,verbose=TRUE,check=TRUE){
  data<-as.data.frame(data)
  
  # process errorhandling object
  errorhandling<-checkerrorhandling(errorhandling)
  
  # check input
  if(check){
    datachecker(data=data,subjvar=subjvar,diffvars=diffvars,stratvars=stratvars,
                subscorevar=subscorevar,aggvar=aggvar,
                errorhandling=errorhandling,standardize=standardize)
  }
  
  # Arrange properly
  runorder<-do.call(order,data[c(subjvar,subscorevar,errorhandling$blockvar,diffvars,stratvars)])
  arr.ds<- data[runorder,c(subjvar,subscorevar,errorhandling$blockvar,diffvars,stratvars,
                           aggvar,errorhandling$errorvar)]
  rm(data)
  origpps<-funique(arr.ds[[subjvar]])
  arr.ds[c(subjvar,subscorevar,diffvars,stratvars)]<-
    cols2ids(arr.ds[c(subjvar,subscorevar,diffvars,stratvars)])
  
  # Create indices
  arr.ds[[".diffidx"]]<-do.call(paste,arr.ds[,c(subjvar,subscorevar,diffvars),drop=FALSE])
  arr.ds[[".subscore"]]<-do.call(paste,arr.ds[,c(subjvar,subscorevar),drop=FALSE])
  subscores<-funique(arr.ds[[".subscore"]])
  pps<-funique(arr.ds[[subjvar]])
  
  diffidx<-funique(arr.ds[,c(subjvar,subscorevar,diffvars,".subscore",".diffidx"),drop=FALSE])
  subscorelist<-split(arr.ds,arr.ds[[".subscore"]])[subscores]
  difflist<-split(arr.ds,arr.ds[[".diffidx"]])[funique(diffidx[[".diffidx"]])]
  
  # Compute splithalf masks by subscore
  keys<-setNames(vector(mode="list",length=length(subscores)),subscores)
  if(verbose){
    cat("Generating splits...\n")
    pb <- txtProgressBar(min = 0L, max = length(subscores), initial = 0L)
  }
  for(ss in subscores){
    if(verbose){ setTxtProgressBar(pb,which(ss==subscores)) }
    iterds<-subscorelist[[ss]][,c(diffvars,stratvars),drop=FALSE]
    grsizes<-runlengths(do.call(paste,args=iterds))
    if(length(grsizes)==0L){grsizes<-nrow(iterds)}
    keys[[ss]]<-stratifiedItersplits(splits=splits, groupsizes=grsizes)
  }
  antikeys<-lapply(keys,function(x) !x)
  if(verbose){close(pb)}
  
  # Generate aggvar matrix and apply error rule
  if(errorhandling$type!="none"){
    aggdim<-"matrix"
    arr.ds[[".blockidx"]]<-
      do.call(paste,arr.ds[,c(subjvar,subscorevar,errorhandling$blockvar),drop=FALSE])
    blockidx<-funique(arr.ds[[".blockidx"]])
    aggmats<-split(x=arr.ds[[aggvar]],f=arr.ds[[".blockidx"]])[blockidx] |> 
      lapply(function(x){ matrix(x,nrow=length(x),ncol=splits)})
    
    # Apply error rule
    if(verbose){
      cat("Replacing error trials\n")
      pb <- txtProgressBar(min = 0L, max = length(blockidx), initial = 0L)
    }
    for(i in seq_along(blockidx)){
      if(verbose){ setTxtProgressBar(pb,i) }
      
      # get mask
      ss<-arr.ds[[".subscore"]][arr.ds[[".blockidx"]]==blockidx[i]][1L]
      maskkey<-arr.ds[arr.ds[[".subscore"]]==ss,".blockidx"]==blockidx[i]
      errorvec<-as.logical(arr.ds[[errorhandling$errorvar]][arr.ds[[".blockidx"]]==blockidx[i]])
      
      # replace errors in key
      aggmats[[blockidx[i]]] <- 
        ReplaceErrorsFixed(x=aggmats[[blockidx[i]]],
                           mask=keys[[ss]][maskkey,],
                           error=errorvec,
                           penalty=errorhandling[["fixedpenalty"]])
      
      # replace errors in antikey
      aggmats[[blockidx[i]]] <- 
        ReplaceErrorsFixed(x=aggmats[[blockidx[i]]],
                           mask=antikeys[[ss]][maskkey,],
                           error=errorvec,
                           penalty=errorhandling[["fixedpenalty"]])
    }
    # Merge to align with arr.ds
    aggmat<-do.call(rbind,aggmats)
    rm(aggmats)
    if(verbose){close(pb)}
    
  }else{
    aggdim<-"vector"
  }
  
  # Select aggregator type
  if(all(is.function(aggfunc))){
    custom.aggfunc<-aggfunc
    aggfunc<-"custom"
  }else{
    aggfunc<-match.arg(aggfunc)
  }
  if(aggdim=="vector"){
    if(aggfunc=="means"){
      aggfunc<-meansByMask
    }else if(aggfunc=="medians"){
      aggfunc<-mediansByMask
    }else if(aggfunc=="custom"){
      aggfunc<-makeCustomAggregator(custom.aggfunc)
    }
  }else if(aggdim=="matrix"){
    if(aggfunc=="means"){
      aggfunc<-colMeansMasked
    }else if(aggfunc=="medians"){
      aggfunc<-colMediansMasked
    }else if(aggfunc=="custom"){
      aggfunc<-makeCustomAggregator2(custom.aggfunc)
    }
  }
  
  # Get aggregates per condition
  keymeans<-antikeymeans<-matrix(ncol=splits,nrow=length(difflist))
  if(verbose){
    cat("Aggregating conditions...\n")
    pb <- txtProgressBar(min = 0L, max = length(difflist), initial = 0L) 
  }
  rownames(keymeans)<-rownames(antikeymeans)<-names(difflist)
  for(i in seq_along(difflist)){
    if(verbose){ setTxtProgressBar(pb,i) }
    ssid<-difflist[[i]][[".subscore"]][1L]
    diffid<-names(difflist)[i]
    
    if(aggdim=="vector"){
      keymeans[i,]<-
        aggfunc(x=difflist[[i]][[aggvar]],
                mask=keys[[ssid]][subscorelist[[ssid]][[".diffidx"]]==diffid,,drop=FALSE])
      antikeymeans[i,]<-
        aggfunc(x=difflist[[i]][[aggvar]],
                mask=antikeys[[ssid]][subscorelist[[ssid]][[".diffidx"]]==diffid,,drop=FALSE])
      
    }else{
      diffkey<-arr.ds[[".diffidx"]]==diffid
      keymeans[i,]<-
        aggfunc(x=aggmat[diffkey,],
                mask=keys[[ssid]][subscorelist[[ssid]][[".diffidx"]]==diffid,,drop=FALSE])
      antikeymeans[i,]<-
        aggfunc(x=aggmat[diffkey,],
                mask=antikeys[[ssid]][subscorelist[[ssid]][[".diffidx"]]==diffid,,drop=FALSE])
    }
  }
  if(aggdim=="matrix"){ rm(aggmat) }
  if(verbose){close(pb)}
  
  # Get scores
  if(length(diffvars)>0L){
    diffidx[[".valence"]]<-Reduce(`*`,x=clamp.range(diffidx[diffvars]))
    keyscores<-antikeyscores<-matrix(ncol=splits,nrow=length(subscores))
    rownames(keyscores)<-rownames(antikeyscores)<-subscores
    for(i in seq_along(subscores)){
      keyscores[i,]<-
        colSums(keymeans[diffidx[[".subscore"]]==subscores[i],,drop=FALSE] * 
                  diffidx[[".valence"]][diffidx[[".subscore"]]==subscores[i]])
      
      antikeyscores[i,]<-
        colSums(antikeymeans[diffidx[[".subscore"]]==subscores[i],,drop=FALSE] * 
                  diffidx[[".valence"]][diffidx[[".subscore"]]==subscores[i]])
    }
  }else{
    keyscores<-keymeans
    antikeyscores<-antikeymeans
  }
  
  # Standardize if necessary
  if(standardize){
    keysds<-antikeysds<-matrix(ncol=splits,nrow=length(subscorelist))
    rownames(keysds)<-rownames(antikeysds)<-subscores
    if(verbose){
      cat("Computing standard deviations to standardize scores by...\n")
      pb <- txtProgressBar(min = 0L, max = length(subscorelist), initial = 0L) 
    }
    for(ss in subscores){
      if(verbose){ setTxtProgressBar(pb,which(ss==subscores)) }
      keysds[ss,] <- sdsByMask(x=subscorelist[[ss]][[aggvar]], mask=keys[[ss]])
      antikeysds[ss,] <- sdsByMask(x=subscorelist[[ss]][[aggvar]], mask=antikeys[[ss]])
    }
    keyscores<-keyscores/keysds
    antikeyscores<-antikeyscores/antikeysds
    if(verbose){ close(pb) }
  }
  
  # Aggregate into single scores, if there were subscores
  if(length(subscorevar)>0L){
    newkeyscores<-newantikeyscores<-matrix(ncol=splits,nrow=length(pps))
    rownames(newkeyscores)<-rownames(newantikeyscores)<-pps
    for(i in seq_along(pps)){
      ss<-funique(arr.ds[[".subscore"]][arr.ds[[subjvar]]==pps[i]])
      newkeyscores[i,]<-colMeans(keyscores[ss,,drop=FALSE])
      newantikeyscores[i,]<-colMeans(antikeyscores[ss,,drop=FALSE])
    }
    keyscores<-newkeyscores
    antikeyscores<-newantikeyscores
  }
  
  # Rename matrix rows to fit names in dataset
  rownames(keyscores)<-origpps[fmatch(rownames(keyscores),pps)]
  rownames(antikeyscores)<-origpps[fmatch(rownames(antikeyscores),pps)]
  
  # Get correlations
  coraggs<-corStatsByColumns(keyscores,antikeyscores)
  
  # Get other stats
  sampsize<-length(pps)
  testsize<-mean(sapply(pps,\(x)sum(arr.ds[[subjvar]]==x)))
  
  # Form output
  out<-list(r=spearmanBrown(coraggs$cormean),
            ci=quantile(spearmanBrown(coraggs$allcors),c(.025,.975)),
            allcors=spearmanBrown(coraggs$allcors),
            nobs=c(sampsize=sampsize,testsize=testsize),
            rcomponents=list(half1var=coraggs$meanxvar,
                             half2var=coraggs$meanyvar,
                             covar=coraggs$meancovar))
  
  # Add individual split halves if requested
  if(include.scores){
    out$scores<-list(half1=keyscores,
                     half2=antikeyscores)
  }
  
  # Add class
  out<-structure(out,class="rapidsplit")
  
  # return
  return(out)
}

#' @param x \code{rapidsplit} object to print or plot.
#' @param goal_r A goal reliability value, which will be used to compute the required test size.
#' @param ... Ignored.
#' @export
#' @rdname rapidsplit
print.rapidsplit<-function(x,goal_r=.80,...){
  coefstr<-paste0("Full-length reliability (Spearman-Brown coefficient):\n",
                  "rSB (",mf(x$nobs[1]-2),") = ",mf(x$r),
                  ", 95%CI [", mf(x$ci[1]), 
                  ", ", mf(x$ci[2]),"]",
                  ", p = ",mf(r2p(x$r,x$nobs[1]),digits=3L),
                  ", based on ",length(x$allcors)," permutations","\n")
  cat(coefstr,sep="")
  rss<-requiredTestSize(obs_r=c(x$r,x$ci),n=x$nobs[2],goal_r=goal_r)
  sizestr<-paste0("To achieve r = ",mf(goal_r),", the required test size is ",
                  mf(rss[1],0),", 95%CI [",mf(rss[3],0),", ",mf(rss[2],0),"]")
  cat(sizestr,sep="")
}

#' @param type Character argument indicating what should be plotted. 
#' By default, this plots the random split whose correlation is closest to the average.
#' However, this can also plot the random split with 
#' the \code{"minimum"} or \code{"maximum"} split-half correlation, or any \code{"random"} split. 
#' \code{"all"} splits can also be plotted together in one figure, while
#' \code{"many"} implies 1000 splits (or less, in case less than 1000 were computed).
#' @param show.labels Should participant IDs be shown above their points in the scatterplot?
#' Defaults to \code{TRUE} and is ignored when \code{type} is \code{"all"}.
#'
#' @export
#' @rdname rapidsplit
plot.rapidsplit<-function(x,type=c("average","minimum","maximum","random","all"),
                          show.labels=TRUE,...){
  if(!("scores" %in% names(x))){
    message("Individual split-half scores were not included in rapidsplit object, so cannot be plotted")
    return()
  }
  
  type<-match.arg(type)
  if(type=="average"){
    title<-"Split-half Scatterplot for\nIteration with Average Reliability"
    idx<-which.min(abs(x$allcors-x$r))
  }else if(type=="minimum"){
    title<-"Split-half Scatterplot for\nIteration with the Lowest Reliability"
    idx<-which.min(x$allcors)
  }else if(type=="maximum"){
    title<-"Split-half Scatterplot for\nIteration with the Highest Reliability"
    idx<-which.max(x$allcors)
  }else if(type=="random"){
    title<-"Split-half Scatterplot for Random Iteration"
    idx<-sample(seq_along(x$allcors),1L)
  }else if(type=="all"){
    title<-"Split-half Scatterplot for All Iterations"
    idx<-seq_along(x$allcors)
  }else if(type=="many"){
    niters<-max(length(x$allcors),1000L)
    title<-paste0("Split-half Scatterplot for ",niters," Iterations")
    idx<-sample(seq_along(x$allcors),niters)
  }
  title <- paste0(title,"\n(r = ", mf(ifelse(length(idx)==1L,x$allcors[idx],x$r)),")")
  
  h1vals<-x$scores$half1[,idx] |> as.vector()
  h2vals<-x$scores$half2[,idx] |> as.vector()
  plot(h1vals,h2vals,pch=20,main=title,
       xlab="Half 1 computed score",ylab="Half 2 computed score",
       col=rgb(0,0,0,ifelse(length(idx)==1L,1,1/sqrt(length(idx)))))
  if(length(idx)==1L && show.labels){
    text(h1vals,h2vals,rownames(x$scores$half1),cex= 0.7, pos=3, offset=0.3)
  }
}



#' @rdname rapidsplit
#'
#' @param split.chunksize,sample.chunksize Number of chunks to divide the splits and sample in 
#' for more memory-efficient computation. This has no bearing on the result.
#'
#' @export
#'
#' @examples
#' 
#' # Compute the reliability of mean RT
#' # in subsets of 200 splits and 100 participants per run
#' rapidsplit.chunks(data=foodAAT,
#'                   subjvar="subjectid",
#'                   aggvar="RT",
#'                   splits=400,
#'                   split.chunksize=200,
#'                   sample.chunksize=50)
#' 
rapidsplit.chunks <-
  function(data,subjvar,aggvar,diffvars=NULL,stratvars=NULL,subscorevar=NULL,
           splits=6000L,
           aggfunc=c("means","medians"),
           errorhandling=list(type=c("none","fixedpenalty"),
                              errorvar=NULL,fixedpenalty=600,blockvar=NULL),
           standardize=FALSE,include.scores=TRUE,verbose=TRUE,check=TRUE,
           split.chunksize=10000L,sample.chunksize=200L){
    
    # process errorhandling object
    errorhandling <- checkerrorhandling(errorhandling)
    
    # check input
    if(check){
      datachecker(data=data,subjvar=subjvar,diffvars=diffvars,stratvars=stratvars,
                  subscorevar=subscorevar,aggvar=aggvar,
                  errorhandling=errorhandling,standardize=standardize)
    }
    
    # check extra args
    stopifnot(is.numeric(sample.chunksize), sample.chunksize > 1)
    stopifnot(is.numeric(split.chunksize), split.chunksize > 0)
    
    # Get split counts
    split.chunks <- c(rep(split.chunksize,times=floor(splits/split.chunksize)),
                      splits %% split.chunksize)
    allpps <- funique(data[[subjvar]])
    sample.chunksizes <- c(rep(sample.chunksize,times=floor(length(allpps)/sample.chunksize)),
                           length(allpps) %% sample.chunksize)
    if(sample.chunksizes[length(sample.chunksizes)]==1){
      sample.chunksizes[length(sample.chunksizes)-1] <- sample.chunksizes[length(sample.chunksizes)-1] + 1
      sample.chunksizes <- sample.chunksizes[seq_len(length(sample.chunksizes)-1L)]
    }
    sample.chunks <- split(allpps,rep(seq_along(sample.chunksizes),times=sample.chunksizes))
  
    # Run split chunks one by one
    outcomes<-list()
    if(verbose){ 
      cat("Running rapidsplit() in chunks...\n")
      pb <- txtProgressBar(min = 0L, max = length(split.chunks)*length(sample.chunks), initial = 0L)  
    }
    for(i in seq_along(split.chunks)){
      splitoutcomes <- list()
      for(j in seq_along(sample.chunks)){
        if(verbose){ setTxtProgressBar(pb,(i-1L)*length(sample.chunks)+j+1L) }
        splitoutcomes[[j]] <-
          rapidsplit(data=data[data[[subjvar]] %fin% sample.chunks[[j]],],
                     subjvar=subjvar,
                     diffvars=diffvars,
                     stratvars=stratvars,
                     subscorevar=subscorevar,
                     aggvar=aggvar,
                     splits=split.chunks[i],
                     aggfunc=aggfunc,
                     errorhandling=errorhandling,
                     standardize=standardize,
                     include.scores=TRUE,
                     verbose=FALSE,
                     check=FALSE)
        
      }
      # Transferring the relevant data to lists without using double the memory
      h1scores <- h2scores <- list()
      for(j in seq_along(splitoutcomes)){
        h1scores[[j]] <- splitoutcomes[[j]]$scores$half1
        splitoutcomes[[j]]$scores$half1 <- NULL
        h2scores[[j]] <- splitoutcomes[[j]]$scores$half2
        splitoutcomes[[j]]$scores$half2 <- NULL
      }
      h1scores <- do.call(rbind,h1scores)
      h2scores <- do.call(rbind,h2scores)
      outcomes[[i]] <- list(corstats=corStatsByColumns(h1scores,h2scores))
      if(include.scores){
        outcomes[[i]]$scores <- list(half1=h1scores,half2=h2scores)
      }
      rm(h1scores,h2scores)
    }
    if(verbose){close(pb)}
    
    # Merge content
    output <- list()
    output[["allcors"]] <- spearmanBrown(unlist(lapply(outcomes,\(x){ x$corstats$allcors })))
    output[["ci"]] <- quantile(output[["allcors"]],c(.025,.975))
    half1var <- mean(unlist(lapply(outcomes,\(x){ x$corstats$xvar })))
    half2var <- mean(unlist(lapply(outcomes,\(x){ x$corstats$yvar })))
    covar <- mean(unlist(lapply(outcomes,\(x){ x$corstats$covar })))
    testsize <- mean(sapply(allpps,\(x)sum(data[[subjvar]]==x)))
    output[["nobs"]] <- c(sampsize=length(allpps),testsize=testsize)
    output[["r"]] <- spearmanBrown(covar/sqrt(half1var*half2var))
    output[["rcomponents"]] <- list(half1var=half1var,
                                    half2var=half2var,
                                    covar=covar)
    
    if(include.scores){
      output[["scores"]][["half1"]]<-do.call(cbind,lapply(outcomes,\(x)x[["scores"]][["half1"]]))
      for(i in seq_along(outcomes)){ outcomes[[i]][["scores"]][["half1"]]<-NULL }
      output[["scores"]][["half2"]]<-do.call(cbind,lapply(outcomes,\(x)x[["scores"]][["half2"]]))
      for(i in seq_along(outcomes)){ outcomes[[i]][["scores"]][["half2"]]<-NULL }
    }
    output<-output[c("r","ci","allcors","nobs","rcomponents","scores")]
    
    # Add class
    output<-structure(output,class="rapidsplit")
    
    # Return
    return(output)
}

#' Approach-Avoidance Task examining approach bias to different foods
#'
#' This data originates from an approach-avoidance task examining approach bias towards food.
#' Participants responded to the stimulus category (food or object) by pulling or pushing a joystick.
#' Instructions were flipped from one block to the next.
#'
#' @docType data
#'
#' @usage data(foodAAT, package="rapidsplithalf")
#'
#' @format An object of class \code{"data.frame"}.
#' 
#' @details
#' * subjectid: Participant ID.
#' * stimid: Stimulus ID.
#' * is_pull: Whether the trial required an approach response (1) or an avoid response (0).
#' * is_target: Whether the trial featured a food stimulus (1) or an object stimulus (0).
#' * error: Whether the response was incorrect (1) or correct (0).
#' * RT: The response initiation time.
#' * FullRT: The time from stimulus onset to response completion.
#' * trialnum: The trial number.
#' * blocknum: The block number.
#' * palatability: The participant's palatability rating for the stimulus (foods only).
#' * valence: The participant's valence rating for the stimulus.
#' * FCQS_2_craving: The participant's FCQS state food craving score at time of testing.
#' * FCQS_2_hunger: The participant's FCQS state hunger score at time of testing.
#' 
#' @md
#'
#' @keywords datasets
#'
#' @references Lender, A., Meule, A., Rinck, M., Brockmeyer, T., & Blechert, J. (2018). 
#' Measurement of food-related approach–avoidance biases: 
#' Larger biases when food stimuli are task relevant. Appetite, 125, 42–47. 
#' \doi{10.1016/j.appet.2018.01.032}
#' 
#' @source \doi{10.1016/j.appet.2018.01.032}
#'
"foodAAT"

#' Implicit Association Task examining implicit bias towards White and Black people
#'
#' This data originates from the publicly available 
#' implicit association test (IAT) on racial prejudice hosted by Project Implicit.
#' 200 participants were randomly sampled from the full trial-level data 
#' available for participants from 2002 to 2022.
#' We included only those IAT blocks relevant to scoring (3,4,6,7) and 
#' only individuals with full data.
#'
#' @docType data
#'
#' @usage data(raceIAT, package="rapidsplithalf")
#'
#' @format An object of class \code{"data.frame"}.
#' 
#' @details
#' * session_id: The session id, proxy for participant number.
#' * task_name: Subtype of IAT used.
#' * block_number: IAT block number.
#' * block_pairing_definition: Stimulus pairing displayed in block.
#' * block_trial_number: Trial number within block.
#' * stimulus: Stimulus name.
#' * required_response: The response required from the participant.
#' * latency: Participant's response latency.
#' * error: Whether the response was wrong (\code{TRUE}).
#' * trial_number: Experimentwise trial number.
#' * stimcat: The stimulus category.
#' * respcat: Category of the required response.
#' * blocktype: Either practice block or full IAT block.
#' * congruent: Whether the block was congruent with anti-black bias (\code{TRUE}) or not.
#' * latency2: Response latencies with those for incorrect responses
#' replaced by the block mean plus a penalty.
#' 
#' @md
#'
#' @keywords datasets
#'
#' @references Xu, K., Nosek, B., & Greenwald, A. G. (2014). 
#' Psychology data from the race implicit association test on the project implicit demo website. 
#' Journal of open psychology data, 2(1), e3-e3. 
#' \doi{10.5334/jopd.ac}
#' 
#' @source \href{https://osf.io/y9hiq/}{OSF.io repository}
#'
"raceIAT"

# library(magrittr)
# library(dplyr)
# load("./../alldata.rda")
# foodAAT<-datasets$relfoo
# foodAAT %<>% select(subjectid,stimid,is_pull,is_target,error,RT,FullRT,trialnum,blocknum,
#                     palatability,valence,FCQS_2_craving,FCQS_2_hunger)
# save(foodAAT,file="./data/foodAAT.RData")

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' stratifiedItersplits
#' 
#' Generate stratified splits for a single participant
#' 
#' This equally splits what can be equally split within groups.
#' Then it randomly splits all the leftovers to ensure near-equal split sizes.
#' This function is moreso used internally, 
#' but you can use it if you know what you are doing.
#'
#' @param splits Number of iterations.
#' @param groupsizes An integer vector of how many RTs per group need to be stratified.
#' @returns A matrix with zeroes and ones. Each column is a random split.
#' @examples
#' 
#' # We will create splits stratified by stimulus for a single participant
#' data(foodAAT)
#' currdata<-foodAAT[foodAAT$subjectid==3,]
#' currdata$stratfactor<-interaction(currdata$is_pull,currdata$is_target,currdata$stimid)
#' currdata<-currdata[order(currdata$stratfactor),]
#' groupsizes<-rle(as.character(currdata$stratfactor))$lengths
#' 
#' mysplits<-stratifiedItersplits(splits=1000,groupsizes=groupsizes)
#' 
#' # Now the data can be split with the values from any column.
#' half1<-currdata[mysplits[,1]==1,]
#' half2<-currdata[mysplits[,1]==0,]
#' 
#' # Or the split objects can be used as masks for the aggregation functions in this package
#' meansByMask(x=currdata$RT,mask=mysplits==1)
#'  
#' @export
stratifiedItersplits <- function(splits, groupsizes) {
    .Call('_rapidsplithalf_stratifiedItersplits', PACKAGE = 'rapidsplithalf', splits, groupsizes)
}

#' Bootstrap Weights
#' 
#' Create a matrix of bootstrap samples expressed as frequency weights
#'
#' @param size Number of values to bootstrap
#' @param times Number of bootstraps
#' @returns A matrix with bootstrap samples expressed as frequency weights. 
#' Each column represents a single bootstrap iteration and each row represents a case.
#' 
#' @examples
#' # Rapidly compute a bootstrapped median to obtain its standard error
#' myweights<-bootstrapWeights(size=50, times=100)
#' meds<-mediansByWeight(x=rnorm(50),weights=myweights)
#' # SE
#' sd(meds)
#' 
#' @export
bootstrapWeights <- function(size, times) {
    .Call('_rapidsplithalf_bootstrapWeights', PACKAGE = 'rapidsplithalf', size, times)
}

#' Correlate two matrices by column
#' 
#' Correlate each column of 1 matrix with the same column in another matrix
#'
#' @param x,y Matrices whose values to correlate by column.
#' @returns \code{corByColumns()} and \code{corByColumns_mask()} return 
#' a numeric vector of correlations of each pair of columns.
#' 
#' \code{corStatsByColumns()} returns a list with named items:
#' - cormean: the aggregated correlation coefficient of all column pairs (see Details)
#' - allcors: the correlations of each column pair
#' - xvar: the column variances of matrix \code{x}
#' - yvar: the column variances of matrix \code{y}
#' - covar: the covariances of each column pair
#' 
#' @md
#' @details
#' The primary use for these functions is to rapidly compute the correlations
#' between two sets of split-half scores stored in matrix columns.
#' 
#' \code{corStatsByColumns} produces the mean correlation of all column-pairs
#' using the formula \code{mean(covariances) / sqrt(mean(col1variance) * mean(col2variance))}
#' 
#' This method is more accurate than [cormean()] and was suggested by 
#' prof. John Christie of Dalhousie University.
#' 
#' @export
#' @author Sercan Kahveci
#' @examples
#' m1<-matrix((1:9)+rnorm(9),ncol=3)
#' m2<-matrix((9:1)+rnorm(9),ncol=3)
#' corByColumns(m1,m2)
#' 
corByColumns <- function(x, y) {
    .Call('_rapidsplithalf_corByColumns', PACKAGE = 'rapidsplithalf', x, y)
}

#' @rdname corByColumns
#' @param mask Logical matrix marking which data points to include.
#' @export
#' @examples
#' mask<-1-diag(3)
#' corByColumns_mask(m1,m2,mask)
#' 
corByColumns_mask <- function(x, y, mask) {
    .Call('_rapidsplithalf_corByColumns_mask', PACKAGE = 'rapidsplithalf', x, y, mask)
}

#' @rdname corByColumns
#' @export
#' @examples
#' corStatsByColumns(m1,m2)
#' 
corStatsByColumns <- function(x, y) {
    .Call('_rapidsplithalf_corStatsByColumns', PACKAGE = 'rapidsplithalf', x, y)
}

#' Fast matrix column aggregators
#' @name colAggregators
#' 
#' @param x A numeric matrix to compute column aggregates of.
#' 
#' @examples
#' x <- cbind(x1 = 3, x2 = c(4:1, 2:5))
#' colMedians(x)
#' 
#' @seealso \link[base]{colMeans}, \link{mediansByMask}, \link{maskAggregators}
#' @return A numeric vector representing values aggregated by column.
#' @export
#' @author Sercan Kahveci
colMedians <- function(x) {
    .Call('_rapidsplithalf_colMedians', PACKAGE = 'rapidsplithalf', x)
}

#' @rdname colAggregators
#' @export
#' @examples
#' colProds(x)
#' 
colProds <- function(x) {
    .Call('_rapidsplithalf_colProds', PACKAGE = 'rapidsplithalf', x)
}

#' @rdname colAggregators
#' @export
#' @examples
#' colSds(x)
#' 
colSds <- function(x) {
    .Call('_rapidsplithalf_colSds', PACKAGE = 'rapidsplithalf', x)
}

#' @rdname colAggregators
#' @param mask A logical matrix determining which data points to include in 
#' the column-wise aggregations.
#' 
#' @examples
#' mask<-cbind(rep(c(TRUE,FALSE),4),
#'             rep(c(TRUE,FALSE),each=4))
#' colMediansMasked(x,mask)
#' 
#' @export
colMediansMasked <- function(x, mask) {
    .Call('_rapidsplithalf_colMediansMasked', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @rdname colAggregators
#' @export
#' @examples
#' colMeansMasked(x,mask)
#' 
colMeansMasked <- function(x, mask) {
    .Call('_rapidsplithalf_colMeansMasked', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @rdname colAggregators
#' @export
#' @examples
#' colSdsMasked(x,mask)
#' 
colSdsMasked <- function(x, mask) {
    .Call('_rapidsplithalf_colSdsMasked', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @title Multi-mask/weight based aggregators
#' @name maskAggregators
#' @description Methods to aggregate the same vector with different masks or frequency weights.
#' Useful for fast bootstrapping or split-half scoring.
#' A single aggregate value of \code{x} is computed for each column of the mask or weight matrix.
#' 
#' @param x A vector to aggregate over with different masks or weights.
#' @param mask Logical matrix where each column represents a separate vector of masks 
#' to aggregate \code{x} with. Only values marked \code{TRUE} are included in the aggregation.
#' 
#' @examples
#' 
#' # Demonstration of mediansByMask()
#' x<-1:6
#' mask<-rbind(c(TRUE,FALSE,FALSE),
#'             c(TRUE,FALSE,FALSE),
#'             c(FALSE,TRUE,FALSE),
#'             c(FALSE,TRUE,FALSE),
#'             c(FALSE,FALSE,TRUE),
#'             c(FALSE,FALSE,TRUE))
#' mediansByMask(x,mask)
#' 
#' # Compute split-halves for a single 
#' # participant, stratified by stimulus
#' data(foodAAT)
#' currdata<-foodAAT[foodAAT$subjectid==3,]
#' currdata$stratfactor<-
#'   interaction(currdata$is_pull,
#'               currdata$is_target,
#'               currdata$stimid)
#' currdata<-currdata[order(currdata$stratfactor),]
#' groupsizes<-
#'   rle(as.character(currdata$stratfactor))$lengths
#' mysplits<-
#'   stratifiedItersplits(splits=1000,
#'                        groupsizes=groupsizes)
#' 
#' # Median for half 1
#' mediansByMask(currdata$RT,mysplits==1)
#'  
#' @seealso \link{colMedians}, \link{colAggregators}, \link{generateSplits}
#' @return a vector with each value representing an aggregate of the same single input vector 
#' but with different masks or frequency weights applied.
#' @export
#' @author Sercan Kahveci
mediansByMask <- function(x, mask) {
    .Call('_rapidsplithalf_mediansByMask', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @rdname maskAggregators
#' @examples
#' #How to use meansByMask()
#' meansByMask(x,mask)
#' sd(meansByMask(currdata$RT,mysplits==1))
#' 
#' @export
meansByMask <- function(x, mask) {
    .Call('_rapidsplithalf_meansByMask', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @rdname maskAggregators
#' @export
#' @examples
#' # How to use sdsByMask() to compute
#' # mask-based D-scores
#' meansByMask(currdata$RT,mysplits==1) / 
#'   sdsByMask(currdata$RT,mysplits==1)
#' 
sdsByMask <- function(x, mask) {
    .Call('_rapidsplithalf_sdsByMask', PACKAGE = 'rapidsplithalf', x, mask)
}

#' @rdname maskAggregators
#' @param weights Integer matrix where each column represents frequency weights 
#' to weight the aggregation by.
#' @examples
#' # Compute the bootstrapped 
#' # standard error of a median
#' weights<-
#'   bootstrapWeights(size=nrow(currdata),
#'                    times=1000)
#' bootmeds<-mediansByWeight(currdata$RT,weights)
#' sd(bootmeds) # bootstrapped standard error
#' 
#' @export
mediansByWeight <- function(x, weights) {
    .Call('_rapidsplithalf_mediansByWeight', PACKAGE = 'rapidsplithalf', x, weights)
}

#' @rdname maskAggregators
#' @export
#' @examples
#' # Compute the bootstrapped 
#' # standard error of a mean
#' bootmeans<-meansByWeight(currdata$RT,weights)
#' sd(bootmeans) # bootstrapped standard error
#' # exact standard error for comparison
#' sd(currdata$RT)/sqrt(length(currdata$RT)) 
#' 
meansByWeight <- function(x, weights) {
    .Call('_rapidsplithalf_meansByWeight', PACKAGE = 'rapidsplithalf', x, weights)
}

#' @rdname maskAggregators
#' @export
#' @examples
#' # Use sdsByWeight to compute bootstrapped D-scores
#' bootsds<-sdsByWeight(currdata$RT,weights)
#' # bootstrapped standard error of D-score
#' sd(bootmeans/bootsds)
#' 
sdsByWeight <- function(x, weights) {
    .Call('_rapidsplithalf_sdsByWeight', PACKAGE = 'rapidsplithalf', x, weights)
}

#' Exclude SD-based outliers
#' 
#' Different masks (columns of a logical matrix) are applied to the same input vector, 
#' and outliers in each resulting subvector are marked with \code{FALSE} in the mask.
#' 
#' @param x Vector to exclude outliers from.
#' @param mask A logical matrix determining which data points to include and which not to.
#' @param sdlim Standard deviation limit to apply; 
#' values beyond are classified as outliers and masked.
#' @returns An updated mask.
#' @export
#' @examples
#' x<-rnorm(50)
#' x[1]<-100
#' x[2]<-50
#' mask<-matrix(TRUE,ncol=3,nrow=50)
#' mask[1,2]<-FALSE
#' mask[2,3]<-FALSE
#' excludeOutliersByMask(x,mask)
excludeOutliersByMask <- function(x, mask, sdlim = 3) {
    .Call('_rapidsplithalf_excludeOutliersByMask', PACKAGE = 'rapidsplithalf', x, mask, sdlim)
}

#' Exclude SD-based outliers in each matrix column
#' 
#' Generate or update a mask matrix based on outlyingness of values in each column.
#' @name OutlierMaskers
#' @param x Matrix in which to mark SD-based outliers by column.
#' @param sdlim Standard deviation limit to apply; 
#' values beyond are classified as outliers and masked.
#' @returns A logical matrix with outliers (and previously masked values) marked as \code{FALSE}.
#' @export
#' @examples
#' # Generate data with outliers
#' testmat<-matrix(rnorm(100),ncol=2)
#' testmat[1,]<-100
#' testmat[2,]<-50
#' 
#' # Detect outliers
#' maskOutliers(testmat)
#' 
maskOutliers <- function(x, sdlim = 3) {
    .Call('_rapidsplithalf_maskOutliers', PACKAGE = 'rapidsplithalf', x, sdlim)
}

#' @rdname OutlierMaskers
#' @param mask A logical matrix determining which data points to include and which not to.
#' @examples
#' # Generate a mask
#' testmask<-matrix(TRUE,ncol=2,nrow=50)
#' testmask[1,1]<-FALSE
#' 
#' # Detect outliers with pre-existing mask
#' maskOutliersMasked(x=testmat, 
#'                    mask=testmask, sdlim = 3)
#' 
#' @export
maskOutliersMasked <- function(x, mask, sdlim = 3) {
    .Call('_rapidsplithalf_maskOutliersMasked', PACKAGE = 'rapidsplithalf', x, mask, sdlim)
}

ReplaceErrorsFixed <- function(x, mask, error, penalty) {
    .Call('_rapidsplithalf_ReplaceErrorsFixed', PACKAGE = 'rapidsplithalf', x, mask, error, penalty)
}

#' A balanced split-half generator
#' 
#' Generates split-half indices that can be stratified by multiple subgroup variables
#' while guaranteeing near-equal numbers of trials in both halves.
#'
#' @param data A dataset to generate split-halves from.
#' @param subsetvars Variables identifying subgroups that must be individually split 
#' into equally sized halves, e.g. participant number and experimental condition.
#' @param stratvars Variables identifying subgroups that are nested within the subsetvars, 
#' and must be split as fairly as possible, while preserving the equal size of 
#' the two halves of each subset identified by the subsetvars, e.g. stimulus ID.
#' @param splits How many splits to generate.
#' @param verbose Display progress bar?
#'
#' @return A logical \code{matrix} in which each row represents a row of the input dataset,
#' and each column represents a single split. 
#' @export
#'
#' @examples
#' data(foodAAT)
#' mysplits<-generateSplits(data=foodAAT,
#'                          subsetvars=c("subjectid","is_pull","is_target"),
#'                          stratvars="stimid",
#'                          splits=1)
#' half1<-foodAAT[ mysplits[,1],]
#' half2<-foodAAT[!mysplits[,1],]
#' 
generateSplits<-function(data,subsetvars,stratvars=NULL,splits=6000,verbose=TRUE){
  
  # Arrange properly
  runorder<-do.call(order,data[,c(subsetvars,stratvars),drop=FALSE])
  arr.ds<- cols2ids(data[runorder,c(subsetvars,stratvars),drop=FALSE])
  
  subsetvec<-do.call(paste,arr.ds[,subsetvars,drop=FALSE])
  subsets<-unique(subsetvec)
  
  # Stratify by split stratum
  origkeys<-setNames(vector(mode="list",length=length(subsets)),subsets)
  if(verbose){ pb <- txtProgressBar(min = 0L, max = length(subsets), initial = 0L) }
  for(ss in subsets){
    if(verbose){ setTxtProgressBar(pb,which(ss==subsets)) }
    iterds<-arr.ds[subsetvec==ss,c(stratvars),drop=FALSE]
    iterrle<-rle(do.call(paste,args=iterds))
    grsizes<-iterrle$lengths
    if(length(grsizes)==0L){grsizes<-nrow(iterds)}
    origkeys[[ss]]<-stratifiedItersplits(splits=splits, groupsizes=grsizes)
  }
  origkeys<-do.call(rbind,origkeys)
  if(verbose){ close(pb) }
  
  # return
  backorder<-order(runorder)
  return(origkeys[backorder,,drop=FALSE])
}

applyAggregator<-function(data,subsetvars,aggvar,aggfunc,mask,verbose=TRUE){
  subsetvec<-do.call(paste,cols2ids(data[,subsetvars,drop=FALSE]))
  runorder<-order(subsetvec)
  subsetvec<-subsetvec[runorder]
  data<-data[runorder,]
  mask<-mask[runorder,]
  
  subsets<-unique(subsetvec)
  aggs<-matrix(ncol=ncol(mask),nrow=length(subsets))
  if(verbose){ pb <- txtProgressBar(min = 0L, max = length(subsets), initial = 0L) }
  for(i in seq_along(subsets)){
    if(verbose){ setTxtProgressBar(pb,i) }
    aggs[i,]<-
      aggfunc(x=data[[aggvar]][subsetvec==subsets[i]],
              mask=mask[subsetvec==subsets[i],,drop=FALSE])
  }
  if(verbose){ close(pb) }
  list(indices=unique(data[,subsetvars]),aggs=aggs)
}

applyAggregator2<-function(data,subsetvars,aggvar,aggfunc,masks,verbose=TRUE){
  outputs<-list()
  for(i in seq_along(masks)){
    outputs[[i]]<-applyAggregator(data,subsetvars,aggvar,aggfunc,masks[[i]],verbose)
  }
  outputs
}

reduceRows<-function(indices,aggs,groupvars,collapsevar,
                     action=c("subtract","add","divide","multiply"),
                     highlow=TRUE){
  action<-match.arg(action)
  
  subsetvec<-do.call(paste,indices[,c(groupvars,collapsevar),drop=FALSE])
  if(is.unsorted(subsetvec)){
    runorder<-order(subsetvec)
    indices<-indices[runorder,]
    aggs<-aggs[runorder,]
  }
  
  groupvec<-do.call(paste,indices[,c(groupvars),drop=FALSE])
  
  # add
  if(action=="add" | action=="multiply"){
    subsets<-unique(groupvec)
    newagg<-matrix(ncol=ncol(aggs),nrow=length(subsets))
    agger<-ifelse(action=="add",colSums,colProds)
    for(i in seq_along(subsets)){
      newagg[i,]<-agger(aggs[groupvec==subsets[i],])
    }
    newindices<-unique(indices[,c(groupvars),drop=FALSE])
  }else{
    # Checks
    condcounts<-table(groupvec)
    if(!all(sapply(unique(groupvec),
                   function(x){length(unique(indices[groupvec==x,collapsevar]))}) == 2)){
      stop("More or less than 2 levels of collapse variable detected.")
    }
    
    if(any(condcounts!=2)){
      stop("In ",sum(condcounts!=2)," case(s) there are less or more than 2 instances",
           " of a permutation of grouping variables.",
           " Cannot unambiguously match the collapse variable.")
    }
    if(highlow){
      mat1<-aggs[indices[[collapsevar]]==max(indices[[collapsevar]]),]
      mat2<-aggs[indices[[collapsevar]]==min(indices[[collapsevar]]),]
    }else{
      mat1<-aggs[indices[[collapsevar]]==min(indices[[collapsevar]]),]
      mat2<-aggs[indices[[collapsevar]]==max(indices[[collapsevar]]),]
    }
    if(action=="subtract"){ newagg<-mat1-mat2 }else
    if(action=="divide"){ newagg<-mat1/mat2 }
    newindices<-indices[indices[[collapsevar]]==max(indices[[collapsevar]]),
                        colnames(indices)!=collapsevar]
  }
  list(indices=newindices,aggs=newagg)
}

#' @name correlation-tools
#' @title Miscellaneous correlation tools
#' @description Helper functions to compute important statistics from correlation coefficients.
#' @param r,r1,r2 Correlation values.
#' @param z Z-scores.
#' @param t t-scores.
#' @param n,n1,n2 Sample sizes.
#' @param alpha The significance level to use.
#' @param x A \code{compcorr} object to print.
#' @param ... Ignored.
#' @seealso \link{cormean}
#' @author Sercan Kahveci
#' @return For \code{r2z()}, \code{z2r}, \code{r2t}, \code{t2r}, and \code{r2p}, 
#' a numeric vector with the requested transformation applied. 
#' For \code{rconfint()}, a numeric vector with two values representing 
#' the lower and upper confidence intervals of the correlation coefficient.
#' For \code{compcorr()}, a \code{compcorr} object containing
#' a z and p value for the requested comparison, 
#' which can be printed with \code{print.compcorr()}.
#' @note \code{compcorr()} should not be used to compare permutation-based reliabilities.
#' In that context, it has an excessive type-1 error. Use [comprel()] instead.
#' @examples
#' z <- r2z(.5)
#' r <- z2r(z)
#' t<-r2t(r,30)
#' r<-t2r(t,30)
#' r2p(r,30)
#' print(rconfint(r,30))
#' print(compcorr(.5,.7,20,20))
NULL

#' @export
#' @describeIn correlation-tools Converts correlation coefficients to z-scores.
r2z<-function(r){
  #.5 * (log(1+r) - log(1-r))
  atanh(r)
}

#' @export
#' @describeIn correlation-tools Converts z-scores to correlation coefficients.
z2r<-function(z){
  # r<-(exp(2*z)-1)/(exp(2*z)+1)
  # rma<-which(is.nan(r) & !is.nan(z))
  # r[rma]<-ifelse(z[rma]>0,1,-1)
  # return(r)
  tanh(z)
}

#' @export
#' @describeIn correlation-tools Converts correlation coefficients to t-scores.
r2t<-function(r,n){ 
  (r*sqrt(n-2))/sqrt(1-r^2)
}

#' @export
#' @describeIn correlation-tools Converts t-scores to correlation coefficients.
t2r<-function(t,n){ 
  t/sqrt(t^2+n-2)
}

#' @export
#' @describeIn correlation-tools Computes the two-sided p-value for a given correlation.
r2p<-function(r,n){ 
  2*pt(abs(r2t(r,n)),n-2,lower.tail=FALSE)
}

#' @export
#' @describeIn correlation-tools Computes confidence intervals for one or multiple correlation coefficients.
rconfint<-function(r,n,alpha=.05){
  z <- r2z(r)
  zint <- qnorm(1 - alpha/2) * sqrt(1/(n - 3))
  if(length(r)==1L){
    confints <- c(z2r(z - zint), z2r(z + zint))
  }else if(length(r)>1L){
    confints <- cbind(z2r(z - zint), z2r(z + zint))
  }else{
    confints <- NULL
  }
  return(confints)
}

#' @export
#' @describeIn correlation-tools Computes the significance of the difference between two correlation coefficients.
compcorr<-function(r1,r2,n1,n2){
  zval<-abs(r2z(r1)-r2z(r2)) / sqrt((1/(n1-3)) + (1/(n2-3)))
  pval<-min(1,pnorm(abs(zval),lower.tail=FALSE)*2)
  return(structure(list(zscore=zval,pvalue=pval),class="compcorr"))
}

#' @export
#' @describeIn correlation-tools Computes the significance of the difference between two correlation coefficients.
print.compcorr<-function(x,...){
  cat("Two-tailed Z-test for the difference between two correlation coefficients.",
      "\nZ =",x$zscore,"\np =",x$pvalue,"\n")
}

#' Compare split-half reliabilities
#'
#' @param x,y Either \code{rapidsplit} objects outputted by [rapidsplithalf()], or 
#' raw vectors or permutation-based split-half reliabilities.
#' @param alternative The type of test to perform: "two.sided", 
#' "less" (x < y), or "greater" (x > y).
#'
#' @returns The p-value for the difference.
#' @details
#' For each split-half correlation in \code{x},
#' this computes the percentage of individual split-half correlations in \code{y}
#' that are smaller; these percentages are then averaged and modified in accordance 
#' with the test requested in \code{alternative} to give the p-value.
#' 
#' @note A large number of splits (>10,000) is recommended to get an accurate p-value.
#' 
#' @md
#' @author Sercan Kahveci
#' @export
#'
#' @examples
#' 
#' # Here we will demonstrate that the reliability of the practice trials in the IAT
#' # is higher than the reliability of the non-practice trials.
#' data(raceIAT)
#' rel1 <- rapidsplit(data=raceIAT[raceIAT$blocktype=="practice",],
#'                    subjvar="session_id",diffvars="congruent",
#'                    subscorevar="blocktype",aggvar="latency",splits=1000,standardize=TRUE)
#'                    
#' rel2 <- rapidsplit(data=raceIAT[raceIAT$blocktype=="full",],
#'                    subjvar="session_id",diffvars="congruent",
#'                    subscorevar="blocktype",aggvar="latency",splits=1000,standardize=TRUE)
#' comprel(rel1,rel2,alternative="greater")
#' 
comprel <- function(x,y,alternative=c("two.sided","less","greater")){
  if(inherits(x,"rapidsplit")){ x <- x$allcors }
  if(inherits(y,"rapidsplit")){ y <- y$allcors }
  minlength <- min(length(x),length(y))
  if(minlength<1000L){
    warning("Insufficient split-half iterations in x and/or y.")
  }
  alternative<-match.arg(alternative)
  
  fulldiff <- mean(sapply(x,\(z){mean(y<z)}))
  if(alternative=="two.sided"){
    min(fulldiff,1-fulldiff)*2
  }else if(alternative=="greater"){
    fulldiff
  }else if(alternative=="less"){
    1-fulldiff
  }
}

#' Compute a minimally biased average of correlation values
#'
#' This function computes a minimally biased average of correlation values.
#' This is needed because simple averaging of correlations is negatively biased,
#' and the often used z-transformation method of averaging correlations is positively biased.
#' The algorithm was developed by Olkin & Pratt (1958).
#'
#' @param r A vector containing correlation values/
#' @param n A single value or vector containing sample sizes/
#' @param weights Character. How should the correlations be weighted?
#' \code{none} leads to no weighting, \code{n} weights by sample size,
#' \code{df} weights by sample size minus one.
#' @param type Character. Determines which averaging algorithm to use, 
#' with "OP5" usually being the most accurate.
#' @param na.rm Logical. Should missing values be removed?
#' @param incl.trans Logical. Should the transformed correlations be included?
#'
#' @return An average correlation.
#' @name cormean
#' @export
#' 
#' @references
#' Olkin, I., & Pratt, J. (1958). Unbiased estimation of certain correlation coefficients.
#' The Annals of Mathematical Statistics, 29. https://doi.org/10.1214/aoms/1177706717
#'
#' Shieh, G. (2010). Estimation of the simple correlation coefficient. Behavior Research Methods,
#' 42(4), 906-917. https://doi.org/10.3758/BRM.42.4.906
#'
#' @examples
#' cormean(c(0,.3,.5),c(30,30,60))
#' 
cormean<-function(r,n,weights=c("none","n","df"),
                  type=c("OP5","OP2","OPK"),
                  na.rm=FALSE,incl.trans=FALSE){
  type<-match.arg(type)
  weights<-match.arg(weights)
  
  if(length(r)==1L){
    return(r)
  }else if(length(n)==1L){
    n<-rep(n,length(r))
  }
  
  if(na.rm){
    missing<-which(is.na(r) | is.na(n))
    if(length(missing)>0L){
      r<-r[-missing]
      n<-n[-missing]
    }
  }
  weight<-list(rep(1,times=length(n)),n,n-1)[[1L+(weights=="n")+2L*(weights=="df")]]
  if(length(r)!=length(n)){
    stop("Length of r and n not equal!")
  }
  
  if(any(n<5)){
    stop("This function cannot accurately average correlations when any have n<5")
  }
  
  if(type=="OP5"){
    sizevec<-unique(n)
    lgammachain1<-lgamma(.5+1:5)*2
    lgammachain2<-lgamma(.5)*2
    factorialchain<-factorial(1:5)
    gammalist<-exp(sapply(sizevec,function(nr){
      (lgammachain1 + lgamma(nr/2-1)) - (lgammachain2 + lgamma(nr/2-1+1:5))}))
    corlist<-sapply(seq_along(r),
                    function(i){ r[i]*(1+ sum(gammalist[,match(n[i],sizevec)] *
                                                (1-r[i]^2)^(1:5)/factorialchain))})
  }else if(type=="OPK"){
    corlist<-r*(1+(1-r^2)/(2*(n-(9*sqrt(2)-7)/2)))
  }else if(type=="OP2"){
    corlist<-r*(1+(1-r^2)/(2*(n-2))+(9*(1-r^2)^2)/(8*n*(n-2)))
  }
  rmean<-weighted.mean(x=corlist,w=weight)
  if(incl.trans){
    return(list(r=rmean,trans=corlist))
  }else{
    return(rmean)
  }
}

---
title: "Readme for {rapidsplithalf}"
author: "Sercan Kahveci"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{rapidsplithalf}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
output:
  knitr:::html_vignette
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, 
                      fig.path = "./../man/figures/README-")
```

## Introduction

The R package *rapidsplithalf* computes the permutation-based split-half reliability of reaction time tasks quickly and accurately, using a balanced splitting algorithm that ensures both halves are of maximally equal size regardless of how many subdivisions exist, for example due to conditions or stratification. *rapidsplithalf* further allows for stratification of the splits, for example by stimulus, to ensure that the two halves are as similar to each other as possible. [Kahveci et al. (2025)](https://doi.org/10.3758/s13423-024-02597-y) found that this stratification enhances the accuracy of the reliability estimate. Lastly, there is support for performing certain pre-processing operations *after* splitting the data rather than before.

## Getting started

```{r intro}
library(rapidsplithalf)
data(foodAAT)
data(raceIAT)
```

Let's say we have a reaction time task with 2 conditions whose RTs we average and then subtract from each other within each participant. We want to know the reliability of this difference score. What do we do?

[Kahveci et al. (2025)](https://doi.org/10.3758/s13423-024-02597-y) give a number of recommendations based on simulation analyses. 

* Split-half reliability far exceeds in precision compared to Cronbach's alpha. They also find much better accuracy in performing many repeated random splits and averaging the result, rather than performing a single even-odd split.

* Most common studies need up to 5500 splits to accurately detect the split-half reliability.

* The accuracy of the reliability estimate is further improved by stratifying the random splits by stimulus such that both halves feature a near-equal number of trials for each stimulus, thereby ensuring that the two halves resemble replications of each other.

* They recommend computing the reliability of the score that is actually used in analyses of the manuscript, i.e. if a difference score is used in correlational analyses, then the reliability of that difference score should be computed rather than of the means that were subtracted from each other. Likewise, if the data was aggregated using medians, then the reliability of a median-based score should be computed.

### Mean difference score

```{r singlediff}
rel<-
rapidsplit(data=foodAAT,
           subjvar="subjectid", # Subject identifier
           diffvars="is_pull", # The variable indicating the two conditions that need to be subtracted from each other
           stratvars="stimid", # splits are stratified by stimulus ID such that each half features a near-equal number of trials for each stimulus
           aggvar="RT", # Defining the variable to be aggregated
           aggfunc="means", # Method to aggregate each condition: simple means
           splits=5500)

print(rel)
```

We can plot one of the splits from the reliability object to check for outliers.

```{r plotter,fig.asp=1}
plot(rel)
```

Or we can plot all of the splits simultaneously in a single figure.

```{r plotter2,fig.asp=1}
plot(rel,type="all")
```

As you can see, the points fall into a diagonal line, with each line representing a single participant. The participants with a longer line have a more uncertain score.

### Double median difference score

As discussed, if you're using medians to aggregate your conditions, then the reliability of a medin-based score should be computed. We do so here within the context of a double-difference score, common to the approach-avoidance task. This has the following format: (push target - pull target) - (push control - pull control)

```{r doublediff}
rel2<-
rapidsplit(data=foodAAT,
           subjvar="subjectid",
           # We specify 2 diffvars for a double-difference score
           diffvars=c("is_pull","is_target"),
           stratvars="stimid",
           aggvar="RT",
           # We specify the median here
           aggfunc="medians",
           splits=5500)

print(rel2)
```

### IAT D-score

We can also compute the reliability of a D-score from an implicit association task.

```{r iatrel,fig.asp=1}

iatrel<-
  rapidsplit(data=raceIAT,
             subjvar="session_id",
             diffvars="congruent",
             
             # The subscorevar argument specifies that we want to compute
             # multiple scores for each participant, based on subsets of
             # their data, and then average the scores together 
             # as is done in the IAT D-score.
             subscorevar="blocktype",
             aggvar="latency",
             splits=1000,
             
             # the errorhandling argument controls how error trials
             # are replaced with the block mean plus a penalty,
             # as is done in the IAT D-score.
             errorhandling=list(type="fixedpenalty",
                                errorvar="error",
                                fixedpenalty=600,
                                blockvar="block_number"),
             
             # The standardize argument specifies that we want to
             # divide the person's score by the standard deviation of their
             # RTs, as in the IAT D-score.
             standardize=TRUE)

print(iatrel)

plot(iatrel,show.labels=FALSE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piper.R
\name{generateSplits}
\alias{generateSplits}
\title{A balanced split-half generator}
\usage{
generateSplits(
  data,
  subsetvars,
  stratvars = NULL,
  splits = 6000,
  verbose = TRUE
)
}
\arguments{
\item{data}{A dataset to generate split-halves from.}

\item{subsetvars}{Variables identifying subgroups that must be individually split 
into equally sized halves, e.g. participant number and experimental condition.}

\item{stratvars}{Variables identifying subgroups that are nested within the subsetvars, 
and must be split as fairly as possible, while preserving the equal size of 
the two halves of each subset identified by the subsetvars, e.g. stimulus ID.}

\item{splits}{How many splits to generate.}

\item{verbose}{Display progress bar?}
}
\value{
A logical \code{matrix} in which each row represents a row of the input dataset,
and each column represents a single split.
}
\description{
Generates split-half indices that can be stratified by multiple subgroup variables
while guaranteeing near-equal numbers of trials in both halves.
}
\examples{
data(foodAAT)
mysplits<-generateSplits(data=foodAAT,
                         subsetvars=c("subjectid","is_pull","is_target"),
                         stratvars="stimid",
                         splits=1)
half1<-foodAAT[ mysplits[,1],]
half2<-foodAAT[!mysplits[,1],]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{maskAggregators}
\alias{maskAggregators}
\alias{mediansByMask}
\alias{meansByMask}
\alias{sdsByMask}
\alias{mediansByWeight}
\alias{meansByWeight}
\alias{sdsByWeight}
\title{Multi-mask/weight based aggregators}
\usage{
mediansByMask(x, mask)

meansByMask(x, mask)

sdsByMask(x, mask)

mediansByWeight(x, weights)

meansByWeight(x, weights)

sdsByWeight(x, weights)
}
\arguments{
\item{x}{A vector to aggregate over with different masks or weights.}

\item{mask}{Logical matrix where each column represents a separate vector of masks 
to aggregate \code{x} with. Only values marked \code{TRUE} are included in the aggregation.}

\item{weights}{Integer matrix where each column represents frequency weights 
to weight the aggregation by.}
}
\value{
a vector with each value representing an aggregate of the same single input vector 
but with different masks or frequency weights applied.
}
\description{
Methods to aggregate the same vector with different masks or frequency weights.
Useful for fast bootstrapping or split-half scoring.
A single aggregate value of \code{x} is computed for each column of the mask or weight matrix.
}
\examples{

# Demonstration of mediansByMask()
x<-1:6
mask<-rbind(c(TRUE,FALSE,FALSE),
            c(TRUE,FALSE,FALSE),
            c(FALSE,TRUE,FALSE),
            c(FALSE,TRUE,FALSE),
            c(FALSE,FALSE,TRUE),
            c(FALSE,FALSE,TRUE))
mediansByMask(x,mask)

# Compute split-halves for a single 
# participant, stratified by stimulus
data(foodAAT)
currdata<-foodAAT[foodAAT$subjectid==3,]
currdata$stratfactor<-
  interaction(currdata$is_pull,
              currdata$is_target,
              currdata$stimid)
currdata<-currdata[order(currdata$stratfactor),]
groupsizes<-
  rle(as.character(currdata$stratfactor))$lengths
mysplits<-
  stratifiedItersplits(splits=1000,
                       groupsizes=groupsizes)

# Median for half 1
mediansByMask(currdata$RT,mysplits==1)
 
#How to use meansByMask()
meansByMask(x,mask)
sd(meansByMask(currdata$RT,mysplits==1))

# How to use sdsByMask() to compute
# mask-based D-scores
meansByMask(currdata$RT,mysplits==1) / 
  sdsByMask(currdata$RT,mysplits==1)

# Compute the bootstrapped 
# standard error of a median
weights<-
  bootstrapWeights(size=nrow(currdata),
                   times=1000)
bootmeds<-mediansByWeight(currdata$RT,weights)
sd(bootmeds) # bootstrapped standard error

# Compute the bootstrapped 
# standard error of a mean
bootmeans<-meansByWeight(currdata$RT,weights)
sd(bootmeans) # bootstrapped standard error
# exact standard error for comparison
sd(currdata$RT)/sqrt(length(currdata$RT)) 

# Use sdsByWeight to compute bootstrapped D-scores
bootsds<-sdsByWeight(currdata$RT,weights)
# bootstrapped standard error of D-score
sd(bootmeans/bootsds)

}
\seealso{
\link{colMedians}, \link{colAggregators}, \link{generateSplits}
}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{foodAAT}
\alias{foodAAT}
\title{Approach-Avoidance Task examining approach bias to different foods}
\format{
An object of class \code{"data.frame"}.
}
\source{
\doi{10.1016/j.appet.2018.01.032}
}
\usage{
data(foodAAT, package="rapidsplithalf")
}
\description{
This data originates from an approach-avoidance task examining approach bias towards food.
Participants responded to the stimulus category (food or object) by pulling or pushing a joystick.
Instructions were flipped from one block to the next.
}
\details{
\itemize{
\item subjectid: Participant ID.
\item stimid: Stimulus ID.
\item is_pull: Whether the trial required an approach response (1) or an avoid response (0).
\item is_target: Whether the trial featured a food stimulus (1) or an object stimulus (0).
\item error: Whether the response was incorrect (1) or correct (0).
\item RT: The response initiation time.
\item FullRT: The time from stimulus onset to response completion.
\item trialnum: The trial number.
\item blocknum: The block number.
\item palatability: The participant's palatability rating for the stimulus (foods only).
\item valence: The participant's valence rating for the stimulus.
\item FCQS_2_craving: The participant's FCQS state food craving score at time of testing.
\item FCQS_2_hunger: The participant's FCQS state hunger score at time of testing.
}
}
\references{
Lender, A., Meule, A., Rinck, M., Brockmeyer, T., & Blechert, J. (2018).
Measurement of food-related approach–avoidance biases:
Larger biases when food stimuli are task relevant. Appetite, 125, 42–47.
\doi{10.1016/j.appet.2018.01.032}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{stratifiedItersplits}
\alias{stratifiedItersplits}
\title{stratifiedItersplits}
\usage{
stratifiedItersplits(splits, groupsizes)
}
\arguments{
\item{splits}{Number of iterations.}

\item{groupsizes}{An integer vector of how many RTs per group need to be stratified.}
}
\value{
A matrix with zeroes and ones. Each column is a random split.
}
\description{
Generate stratified splits for a single participant
}
\details{
This equally splits what can be equally split within groups.
Then it randomly splits all the leftovers to ensure near-equal split sizes.
This function is moreso used internally, 
but you can use it if you know what you are doing.
}
\examples{

# We will create splits stratified by stimulus for a single participant
data(foodAAT)
currdata<-foodAAT[foodAAT$subjectid==3,]
currdata$stratfactor<-interaction(currdata$is_pull,currdata$is_target,currdata$stimid)
currdata<-currdata[order(currdata$stratfactor),]
groupsizes<-rle(as.character(currdata$stratfactor))$lengths

mysplits<-stratifiedItersplits(splits=1000,groupsizes=groupsizes)

# Now the data can be split with the values from any column.
half1<-currdata[mysplits[,1]==1,]
half2<-currdata[mysplits[,1]==0,]

# Or the split objects can be used as masks for the aggregation functions in this package
meansByMask(x=currdata$RT,mask=mysplits==1)
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cormean.R
\name{correlation-tools}
\alias{correlation-tools}
\alias{r2z}
\alias{z2r}
\alias{r2t}
\alias{t2r}
\alias{r2p}
\alias{rconfint}
\alias{compcorr}
\alias{print.compcorr}
\title{Miscellaneous correlation tools}
\usage{
r2z(r)

z2r(z)

r2t(r, n)

t2r(t, n)

r2p(r, n)

rconfint(r, n, alpha = 0.05)

compcorr(r1, r2, n1, n2)

\method{print}{compcorr}(x, ...)
}
\arguments{
\item{r, r1, r2}{Correlation values.}

\item{z}{Z-scores.}

\item{n, n1, n2}{Sample sizes.}

\item{t}{t-scores.}

\item{alpha}{The significance level to use.}

\item{x}{A \code{compcorr} object to print.}

\item{...}{Ignored.}
}
\value{
For \code{r2z()}, \code{z2r}, \code{r2t}, \code{t2r}, and \code{r2p}, 
a numeric vector with the requested transformation applied. 
For \code{rconfint()}, a numeric vector with two values representing 
the lower and upper confidence intervals of the correlation coefficient.
For \code{compcorr()}, a \code{compcorr} object containing
a z and p value for the requested comparison, 
which can be printed with \code{print.compcorr()}.
}
\description{
Helper functions to compute important statistics from correlation coefficients.
}
\section{Functions}{
\itemize{
\item \code{r2z()}: Converts correlation coefficients to z-scores.

\item \code{z2r()}: Converts z-scores to correlation coefficients.

\item \code{r2t()}: Converts correlation coefficients to t-scores.

\item \code{t2r()}: Converts t-scores to correlation coefficients.

\item \code{r2p()}: Computes the two-sided p-value for a given correlation.

\item \code{rconfint()}: Computes confidence intervals for one or multiple correlation coefficients.

\item \code{compcorr()}: Computes the significance of the difference between two correlation coefficients.

\item \code{print(compcorr)}: Computes the significance of the difference between two correlation coefficients.

}}
\note{
\code{compcorr()} should not be used to compare permutation-based reliabilities.
In that context, it has an excessive type-1 error. Use [comprel()] instead.
}
\examples{
z <- r2z(.5)
r <- z2r(z)
t<-r2t(r,30)
r<-t2r(t,30)
r2p(r,30)
print(rconfint(r,30))
print(compcorr(.5,.7,20,20))
}
\seealso{
\link{cormean}
}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spearmanbrown.R
\name{requiredTestSize}
\alias{requiredTestSize}
\title{Required test size for target reliability}
\usage{
requiredTestSize(
  obs_r,
  goal_r = 0.8,
  n = 1,
  fix.negative = c("mirror", "nullify", "none")
)
}
\arguments{
\item{obs_r}{The observed reliability}

\item{goal_r}{The goal reliability}

\item{n}{Current test size, i.e., the number of items or trials.}

\item{fix.negative}{How should negative observed reliabilities be handled? See \code{\link[=spearmanBrown]{spearmanBrown()}}.}
}
\value{
The required test size.
}
\description{
Compute how many items a test needs, for its reliability to reach a goal value.
}
\details{
This function is obtained by solving the \code{\link[=spearmanBrown]{spearmanBrown()}} formula for the \code{ntests} argument.
}
\examples{
requiredTestSize(obs_r=0.5, goal_r=0.8,n=256)

requiredTestSize(obs_r=0, goal_r=0.8,n=256)

}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{corByColumns}
\alias{corByColumns}
\alias{corByColumns_mask}
\alias{corStatsByColumns}
\title{Correlate two matrices by column}
\usage{
corByColumns(x, y)

corByColumns_mask(x, y, mask)

corStatsByColumns(x, y)
}
\arguments{
\item{x, y}{Matrices whose values to correlate by column.}

\item{mask}{Logical matrix marking which data points to include.}
}
\value{
\code{corByColumns()} and \code{corByColumns_mask()} return
a numeric vector of correlations of each pair of columns.

\code{corStatsByColumns()} returns a list with named items:
\itemize{
\item cormean: the aggregated correlation coefficient of all column pairs (see Details)
\item allcors: the correlations of each column pair
\item xvar: the column variances of matrix \code{x}
\item yvar: the column variances of matrix \code{y}
\item covar: the covariances of each column pair
}
}
\description{
Correlate each column of 1 matrix with the same column in another matrix
}
\details{
The primary use for these functions is to rapidly compute the correlations
between two sets of split-half scores stored in matrix columns.

\code{corStatsByColumns} produces the mean correlation of all column-pairs
using the formula \code{mean(covariances) / sqrt(mean(col1variance) * mean(col2variance))}

This method is more accurate than \code{\link[=cormean]{cormean()}} and was suggested by
prof. John Christie of Dalhousie University.
}
\examples{
m1<-matrix((1:9)+rnorm(9),ncol=3)
m2<-matrix((9:1)+rnorm(9),ncol=3)
corByColumns(m1,m2)

mask<-1-diag(3)
corByColumns_mask(m1,m2,mask)

corStatsByColumns(m1,m2)

}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{rapidsplithalf}
\alias{rapidsplithalf}
\alias{rapidsplithalf-package}
\title{rapidsplithalf package}
\description{
Accurately estimates the reliability of cognitive tasks using a fast and flexible 
permutation-based split-half reliability algorithm 
that supports stratified splitting while maintaining equal split sizes. 
See Kahveci, Bathke, and Blechert (2025) \doi{10.3758/s13423-024-02597-y} for details.
To learn more about rapidsplithalf, view the introductory vignette:
\code{vignette("rapidsplithalf",package="rapidsplithalf")}
}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cormean.R
\name{comprel}
\alias{comprel}
\title{Compare split-half reliabilities}
\usage{
comprel(x, y, alternative = c("two.sided", "less", "greater"))
}
\arguments{
\item{x, y}{Either \code{rapidsplit} objects outputted by \code{\link[=rapidsplithalf]{rapidsplithalf()}}, or
raw vectors or permutation-based split-half reliabilities.}

\item{alternative}{The type of test to perform: "two.sided",
"less" (x < y), or "greater" (x > y).}
}
\value{
The p-value for the difference.
}
\description{
Compare split-half reliabilities
}
\details{
For each split-half correlation in \code{x},
this computes the percentage of individual split-half correlations in \code{y}
that are smaller; these percentages are then averaged and modified in accordance
with the test requested in \code{alternative} to give the p-value.
}
\note{
A large number of splits (>10,000) is recommended to get an accurate p-value.
}
\examples{

# Here we will demonstrate that the reliability of the practice trials in the IAT
# is higher than the reliability of the non-practice trials.
data(raceIAT)
rel1 <- rapidsplit(data=raceIAT[raceIAT$blocktype=="practice",],
                   subjvar="session_id",diffvars="congruent",
                   subscorevar="blocktype",aggvar="latency",splits=1000,standardize=TRUE)
                   
rel2 <- rapidsplit(data=raceIAT[raceIAT$blocktype=="full",],
                   subjvar="session_id",diffvars="congruent",
                   subscorevar="blocktype",aggvar="latency",splits=1000,standardize=TRUE)
comprel(rel1,rel2,alternative="greater")

}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rapidsplit.R
\name{rapidsplit}
\alias{rapidsplit}
\alias{print.rapidsplit}
\alias{plot.rapidsplit}
\alias{rapidsplit.chunks}
\title{rapidsplit}
\usage{
rapidsplit(
  data,
  subjvar,
  aggvar,
  diffvars = NULL,
  stratvars = NULL,
  subscorevar = NULL,
  splits = 6000L,
  aggfunc = c("means", "medians"),
  errorhandling = list(type = c("none", "fixedpenalty"), errorvar = NULL, fixedpenalty =
    600, blockvar = NULL),
  standardize = FALSE,
  include.scores = TRUE,
  verbose = TRUE,
  check = TRUE
)

\method{print}{rapidsplit}(x, goal_r = 0.8, ...)

\method{plot}{rapidsplit}(
  x,
  type = c("average", "minimum", "maximum", "random", "all"),
  show.labels = TRUE,
  ...
)

rapidsplit.chunks(
  data,
  subjvar,
  aggvar,
  diffvars = NULL,
  stratvars = NULL,
  subscorevar = NULL,
  splits = 6000L,
  aggfunc = c("means", "medians"),
  errorhandling = list(type = c("none", "fixedpenalty"), errorvar = NULL, fixedpenalty =
    600, blockvar = NULL),
  standardize = FALSE,
  include.scores = TRUE,
  verbose = TRUE,
  check = TRUE,
  split.chunksize = 10000L,
  sample.chunksize = 200L
)
}
\arguments{
\item{data}{Dataset, a \code{data.frame}.}

\item{subjvar}{Subject ID variable name, a \code{character}.}

\item{aggvar}{Name of variable whose values to aggregate, a \code{character}.
Examples include reaction times and error rates.}

\item{diffvars}{Names of variables that determine which conditions
need to be subtracted from each other, \code{character}.}

\item{stratvars}{Additional variables that the splits should
be stratified by; a \code{character}.}

\item{subscorevar}{A \code{character} variable identifying subgroups within
a participant's data from which separate scores should be computed.
To compute a participant's final score, these subscores will be averaged together.
A typical use case is the D-score of the implicit association task.}

\item{splits}{Number of split-halves to average, an \code{integer}.
It is recommended to use around 5000.}

\item{aggfunc}{The function by which to aggregate the variable
defined in \code{aggvar}; can be \code{"means"}, \code{"medians"},
or a custom function (not a function name).
This custom function must take a numeric vector and output a single value.}

\item{errorhandling}{A list with 4 named items, to be used to replace error trials
with the block mean of correct responses plus a fixed penalty, as in the IAT D-score.
The 4 items are \code{type} which can be set to \code{"none"} for no error replacement,
or \code{"fixedpenalty"} to replace error trials as described;
\code{errorvar} requires name of the \code{logical} variable indicating an incorrect response
(as \code{TRUE}); \code{fixedpenalty} indicates how much of a penalty should be added
to said block mean; and \code{blockvar} indicates the name of the block variable.}

\item{standardize}{Whether to divide by scores by the subject's SD; a \code{logical}.
Regardless of whether error penalization is utilized, this standardization
will be based on the unpenalized SD of correct and incorrect trials, as in the IAT D-score.}

\item{include.scores}{Include all individual split-half scores?}

\item{verbose}{Display progress bars? Defaults to \code{TRUE}.}

\item{check}{Check input for possible problems?}

\item{x}{\code{rapidsplit} object to print or plot.}

\item{goal_r}{A goal reliability value, which will be used to compute the required test size.}

\item{...}{Ignored.}

\item{type}{Character argument indicating what should be plotted. 
By default, this plots the random split whose correlation is closest to the average.
However, this can also plot the random split with 
the \code{"minimum"} or \code{"maximum"} split-half correlation, or any \code{"random"} split. 
\code{"all"} splits can also be plotted together in one figure, while
\code{"many"} implies 1000 splits (or less, in case less than 1000 were computed).}

\item{show.labels}{Should participant IDs be shown above their points in the scatterplot?
Defaults to \code{TRUE} and is ignored when \code{type} is \code{"all"}.}

\item{split.chunksize, sample.chunksize}{Number of chunks to divide the splits and sample in 
for more memory-efficient computation. This has no bearing on the result.}
}
\value{
A \code{list} containing
\itemize{
\item \code{r}: the averaged reliability.
\item \code{ci}: the 95\% confidence intervals.
\item \code{allcors}: a vector with the reliability of each iteration.
\item \code{nobs}: a vector with (1) the number of participants and (2) the average number of values per participant.
\item \code{rcomponents}: a list containing the mean variance of the scores of both halves,
as well as their mean covariance.
\item \code{scores}: the individual participants scores in each split-half,
contained in a list with two matrices (Only included if requested with \code{include.scores}).
}
}
\description{
A very fast algorithm for computing stratified permutation-based split-half reliability.
}
\details{
The order of operations (with optional steps between brackets) is:
\itemize{
\item Splitting
\item (Replacing error trials within block within split)
\item Computing aggregates per condition (per subscore) per person
\item Subtracting conditions from each other
\item (Dividing the resulting (sub)score by the SD of the data used to compute that (sub)score)
\item (Averaging subscores together into a single score per person)
\item Computing the covariances of scores from one half with scores from the other half
for every split
\item Computing the variances of scores within each half for every split
\item Computing the average split-half correlation with the average variances and covariance
across all splits, using \code{\link[=corStatsByColumns]{corStatsByColumns()}}
\item Applying the Spearman-Brown formula to the absolute correlation
using \code{\link[=spearmanBrown]{spearmanBrown()}}, and restoring the original sign after
}

\code{\link[=cormean]{cormean()}} was used to aggregate correlations in previous versions
of this package & in the associated manuscript, but the method based on
(co)variance averaging was found to be more accurate. This was suggested by
prof. John Christie of Dalhousie University.
}
\note{
\itemize{
\item \code{\link[=rapidsplit]{rapidsplit()}} function can use a lot of memory in one go.
If you are computing the reliability of a large dataset or you have little RAM,
it may pay off to use \code{\link[=rapidsplit.chunks]{rapidsplit.chunks()}} instead.
\item It is currently unclear it is better to pre-process your data before or after splitting it.
If you are computing the IAT D-score,
you can therefore use \code{errorhandling} and \code{standardize} to perform these two actions
after splitting, or you can process your data before splitting and forgo these two options.
}
}
\examples{

data(foodAAT)
# Reliability of the double difference score:
# {RT(push food)-RT(pull food)} - {RT(push object)-RT(pull object)}

frel<-rapidsplit(data=foodAAT,
                 subjvar="subjectid",
                 diffvars=c("is_pull","is_target"),
                 stratvars="stimid",
                 aggvar="RT",
                 splits=100)
                 
print(frel)

plot(frel,type="average")

           
# Compute a single random split-half reliability of the error rate
rapidsplit(data=foodAAT,
           subjvar="subjectid",
           aggvar="error",
           splits=1,
           aggfunc="means")

# Compute the reliability of an IAT D-score
data(raceIAT)
rapidsplit(data=raceIAT,
           subjvar="session_id",
           diffvars="congruent",
           subscorevar="blocktype",
           aggvar="latency",
           errorhandling=list(type="fixedpenalty",errorvar="error",
                              fixedpenalty=600,blockvar="block_number"),
           splits=10,
           standardize=TRUE)


# Compute the reliability of mean RT
# in subsets of 200 splits and 100 participants per run
rapidsplit.chunks(data=foodAAT,
                  subjvar="subjectid",
                  aggvar="RT",
                  splits=400,
                  split.chunksize=200,
                  sample.chunksize=50)

}
\references{
Kahveci, S., Bathke, A.C. & Blechert, J. (2024)
Reaction-time task reliability is more accurately computed with
permutation-based split-half correlations than with Cronbach’s alpha.
Psychonomic Bulletin and Review. \doi{10.3758/s13423-024-02597-y}
}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{excludeOutliersByMask}
\alias{excludeOutliersByMask}
\title{Exclude SD-based outliers}
\usage{
excludeOutliersByMask(x, mask, sdlim = 3)
}
\arguments{
\item{x}{Vector to exclude outliers from.}

\item{mask}{A logical matrix determining which data points to include and which not to.}

\item{sdlim}{Standard deviation limit to apply; 
values beyond are classified as outliers and masked.}
}
\value{
An updated mask.
}
\description{
Different masks (columns of a logical matrix) are applied to the same input vector, 
and outliers in each resulting subvector are marked with \code{FALSE} in the mask.
}
\examples{
x<-rnorm(50)
x[1]<-100
x[2]<-50
mask<-matrix(TRUE,ncol=3,nrow=50)
mask[1,2]<-FALSE
mask[2,3]<-FALSE
excludeOutliersByMask(x,mask)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spearmanbrown.R
\name{spearmanBrown}
\alias{spearmanBrown}
\title{Spearman-Brown correction
Perform a Spearman-Brown correction on the provided correlation score.}
\usage{
spearmanBrown(r, ntests = 2, fix.negative = c("mirror", "nullify", "none"))
}
\arguments{
\item{r}{To-be-corrected correlation coefficient.}

\item{ntests}{An integer indicating how many times larger the full test is,
for which the corrected correlation coefficient is being computed.}

\item{fix.negative}{How will negative input values be dealt with?
\itemize{
\item \code{"mirror"} submits the absolute correlations to the formula
and restores the original sign afterwards.
\item \code{"nullify"} sets negative correlations to zero.
\item \code{"none"} leaves them as-is (not recommended).
}}
}
\value{
Spearman-Brown corrected correlation coefficients.
}
\description{
Spearman-Brown correction
Perform a Spearman-Brown correction on the provided correlation score.
}
\details{
When \code{ntests=2}, the formula will compute what the correlation coefficient would be
if the test were twice as long.
}
\examples{
spearmanBrown(.5)

}
\seealso{
\code{\link[=requiredTestSize]{requiredTestSize()}} to compute how many items a test with a known reliability needs, to achieve a goal reliability.
}
\author{
Sercan Kahveci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{OutlierMaskers}
\alias{OutlierMaskers}
\alias{maskOutliers}
\alias{maskOutliersMasked}
\title{Exclude SD-based outliers in each matrix column}
\usage{
maskOutliers(x, sdlim = 3)

maskOutliersMasked(x, mask, sdlim = 3)
}
\arguments{
\item{x}{Matrix in which to mark SD-based outliers by column.}

\item{sdlim}{Standard deviation limit to apply; 
values beyond are classified as outliers and masked.}

\item{mask}{A logical matrix determining which data points to include and which not to.}
}
\value{
A logical matrix with outliers (and previously masked values) marked as \code{FALSE}.
}
\description{
Generate or update a mask matrix based on outlyingness of values in each column.
}
\examples{
# Generate data with outliers
testmat<-matrix(rnorm(100),ncol=2)
testmat[1,]<-100
testmat[2,]<-50

# Detect outliers
maskOutliers(testmat)

# Generate a mask
testmask<-matrix(TRUE,ncol=2,nrow=50)
testmask[1,1]<-FALSE

# Detect outliers with pre-existing mask
maskOutliersMasked(x=testmat, 
                   mask=testmask, sdlim = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{bootstrapWeights}
\alias{bootstrapWeights}
\title{Bootstrap Weights}
\usage{
bootstrapWeights(size, times)
}
\arguments{
\item{size}{Number of values to bootstrap}

\item{times}{Number of bootstraps}
}
\value{
A matrix with bootstrap samples expressed as frequency weights. 
Each column represents a single bootstrap iteration and each row represents a case.
}
\description{
Create a matrix of bootstrap samples expressed as frequency weights
}
\examples{
# Rapidly compute a bootstrapped median to obtain its standard error
myweights<-bootstrapWeights(size=50, times=100)
meds<-mediansByWeight(x=rnorm(50),weights=myweights)
# SE
sd(meds)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cormean.R
\name{cormean}
\alias{cormean}
\title{Compute a minimally biased average of correlation values}
\usage{
cormean(
  r,
  n,
  weights = c("none", "n", "df"),
  type = c("OP5", "OP2", "OPK"),
  na.rm = FALSE,
  incl.trans = FALSE
)
}
\arguments{
\item{r}{A vector containing correlation values/}

\item{n}{A single value or vector containing sample sizes/}

\item{weights}{Character. How should the correlations be weighted?
\code{none} leads to no weighting, \code{n} weights by sample size,
\code{df} weights by sample size minus one.}

\item{type}{Character. Determines which averaging algorithm to use, 
with "OP5" usually being the most accurate.}

\item{na.rm}{Logical. Should missing values be removed?}

\item{incl.trans}{Logical. Should the transformed correlations be included?}
}
\value{
An average correlation.
}
\description{
This function computes a minimally biased average of correlation values.
This is needed because simple averaging of correlations is negatively biased,
and the often used z-transformation method of averaging correlations is positively biased.
The algorithm was developed by Olkin & Pratt (1958).
}
\examples{
cormean(c(0,.3,.5),c(30,30,60))

}
\references{
Olkin, I., & Pratt, J. (1958). Unbiased estimation of certain correlation coefficients.
The Annals of Mathematical Statistics, 29. https://doi.org/10.1214/aoms/1177706717

Shieh, G. (2010). Estimation of the simple correlation coefficient. Behavior Research Methods,
42(4), 906-917. https://doi.org/10.3758/BRM.42.4.906
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{raceIAT}
\alias{raceIAT}
\title{Implicit Association Task examining implicit bias towards White and Black people}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{https://osf.io/y9hiq/}{OSF.io repository}
}
\usage{
data(raceIAT, package="rapidsplithalf")
}
\description{
This data originates from the publicly available
implicit association test (IAT) on racial prejudice hosted by Project Implicit.
200 participants were randomly sampled from the full trial-level data
available for participants from 2002 to 2022.
We included only those IAT blocks relevant to scoring (3,4,6,7) and
only individuals with full data.
}
\details{
\itemize{
\item session_id: The session id, proxy for participant number.
\item task_name: Subtype of IAT used.
\item block_number: IAT block number.
\item block_pairing_definition: Stimulus pairing displayed in block.
\item block_trial_number: Trial number within block.
\item stimulus: Stimulus name.
\item required_response: The response required from the participant.
\item latency: Participant's response latency.
\item error: Whether the response was wrong (\code{TRUE}).
\item trial_number: Experimentwise trial number.
\item stimcat: The stimulus category.
\item respcat: Category of the required response.
\item blocktype: Either practice block or full IAT block.
\item congruent: Whether the block was congruent with anti-black bias (\code{TRUE}) or not.
\item latency2: Response latencies with those for incorrect responses
replaced by the block mean plus a penalty.
}
}
\references{
Xu, K., Nosek, B., & Greenwald, A. G. (2014).
Psychology data from the race implicit association test on the project implicit demo website.
Journal of open psychology data, 2(1), e3-e3.
\doi{10.5334/jopd.ac}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{colAggregators}
\alias{colAggregators}
\alias{colMedians}
\alias{colProds}
\alias{colSds}
\alias{colMediansMasked}
\alias{colMeansMasked}
\alias{colSdsMasked}
\title{Fast matrix column aggregators}
\usage{
colMedians(x)

colProds(x)

colSds(x)

colMediansMasked(x, mask)

colMeansMasked(x, mask)

colSdsMasked(x, mask)
}
\arguments{
\item{x}{A numeric matrix to compute column aggregates of.}

\item{mask}{A logical matrix determining which data points to include in 
the column-wise aggregations.}
}
\value{
A numeric vector representing values aggregated by column.
}
\description{
Fast matrix column aggregators
}
\examples{
x <- cbind(x1 = 3, x2 = c(4:1, 2:5))
colMedians(x)

colProds(x)

colSds(x)

mask<-cbind(rep(c(TRUE,FALSE),4),
            rep(c(TRUE,FALSE),each=4))
colMediansMasked(x,mask)

colMeansMasked(x,mask)

colSdsMasked(x,mask)

}
\seealso{
\link[base]{colMeans}, \link{mediansByMask}, \link{maskAggregators}
}
\author{
Sercan Kahveci
}
